var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-879ECFAD-7928-44B3-BCD7-276D53C89B52\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Class and Object Inspector Functions</h1></div><p>MAXScript provides a number of built-in functions for displaying information about the classes that are accessible in your particular <strong>3ds Max</strong> installation. These functions are useful as an online reference for the core <strong>3ds Max</strong> classes and provide the only way of determining what properties in <a href=\"Third-Party-Plug-In-Classes.html\">third-party plug-in classes</a> are accessible.</p>\n\
<p>{toc}</p>\n\
<p><a name=\"pattern-strings\"></a></p>\n\
<h2 id=\"pattern-strings\">Pattern Strings</h2><p>The <code>apropos()</code>, <code>showClass()</code> and <code>showProperties()</code> methods take pattern strings as input.  Pattern strings can contain:</p>\n\
<ul>\n\
<li>regular characters to be matched exactly</li>\n\
<li><code>*</code> matching any sequence of characters (zero or more)</li>\n\
<li><code>?</code> matching any single character</li>\n\
<li><code>[SET]</code> matching any character in the set</li>\n\
<li><code>[!SET]</code> or <code>[^SET]</code> matching any character NOT in the set. </li>\n\
</ul>\n\
<p>A set is composed of characters, or a range of characters separated by a hyphen, such as &quot;0-9&quot; or &quot;A-D&quot;.  Sets are case-sensitive, and uppercase letters are &quot;less than&quot; lowercase, so &quot;a-Z&quot; is not valid, while &quot;A-z&quot; is.</p>\n\
<p>Special characters can be escaped (treated as a literal character) by prepending with a backslash <code>\\</code>.  The special characters are: <code>[]*?!^-\\</code></p>\n\
<h2 id=\"apropos\">apropos</h2><p>The <code>apropos()</code> function is used to search for and print out information about global variables by name pattern and class of contents.</p>\n\
<p>This function has the form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">apropos &lt;pattern_string&gt; [ implicitWild:true ] [ to:&lt;stream&gt; ]</code></pre><p>The <code>&lt;pattern_string&gt;</code> argument is a string containing a wild-card pattern for matching against global variables.</p>\n\
<p>The pattern form is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;variable_name_pattern_string[:class_name_pattern_string]&gt;</code></pre><p>that is, a global variable name pattern optionally followed by a &apos;:&apos; and a class name pattern.</p>\n\
<p>If the optional class name pattern is given, it restricts the printout to globals containing values of the specified classes.</p>\n\
<p>The <code>apropos()</code> function assumes a wild-card &apos;*&apos; at the start and end of each pattern, implying a &apos;contains&apos; pattern match, unless <code>implicitWild:false</code> is specified, in which case no &apos;*&apos; are assumed. See <a href=\"#pattern-strings\">Pattern Strings</a> above for more information.</p>\n\
<p>The output consists of the global variable name, a tag in () indicating if the variable is a system or const (unchangeable) variable and the class of the value currently in it, followed by a printed representation of that value.</p>\n\
<p><strong>EXAMPLES</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">apropos &quot;light&quot; -- any variables with &apos;light&apos; in the variable name\n\
apropos &quot;:float&quot; -- any variables containing float values\n\
apropos &quot;&quot; to:f -- dump all variables to the stream f\n\
apropos &quot;controller:super&quot; -- the list of controller superclasses\n\
apropos &quot;foo:control&quot; -- any variables whose name contains &apos;foo&apos;\n\
-- and that contain controller classes\n\
&gt;   apropos &quot;[x-z]*&quot; implicitWild:false -- any classes or functions that start with x, y, or z\n\
</code></pre><p>Here&apos;s a fragment of the first example&apos;s output:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">lights (const ObjectSet): $lights\n\
lightLevel (system Float): 1.0\n\
Sunlight (const MAXClass): Sunlight\n\
Directionallight (const MAXClass): Directionallight\n\
lightLevelController (system Control): Controller:Bezier_Float\n\
light (const MAXSuperClass): light\n\
NewLight (const Primitive): NewLight()\n\
lightTintColor (system Color): (color 255 255 255)\n\
Volume_Light (const MAXClass): Volume_Light\n\
lightTintColorController (system Control): Controller:Bezier_Color\n\
gw.getMaxLights (Primitive): getMaxLights()\n\
gw.setLight (Primitive): setLight()\n\
</code></pre><p>To make scripting easier to teach and use everyday, a ScriptTool function for <strong>Help</strong> has been added. It is an alias for <strong>apropos</strong>.</p>\n\
<p>Typing <strong>Help &quot;Box&quot;</strong> will show all the commands using the name BOX.</p>\n\
<h2 id=\"showclass\">showClass</h2><p>The <code>showClass()</code> function prints information about a specified <strong>3ds Max</strong> class or classes. It has the following form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showClass &lt;pattern_string&gt; [ to:&lt;stream&gt; ]</code></pre><p>where <code>&lt;pattern_string&gt;</code> is a string containing a wild-card pattern to match against <strong>3ds Max</strong> class names, superclass names and property names, and the optional <code>to:&lt;stream&gt;</code> keyword argument specifies a <a href=\"./../../MAXScript-Language-Reference/Values/Stream-Values/Stream-Values-1.html\">Stream Value</a> to output the display to.</p>\n\
<p>The pattern string has this form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&quot;&lt;class_name&gt;[:&lt;superclass_name&gt;][.&lt;property_name&gt;]&quot;</code></pre><p>See <a href=\"#pattern-strings\">Pattern Strings</a> above for more information about special characters allowed in pattern strings.</p>\n\
<p><strong>EXAMPLES</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showClass &quot;path*&quot; -- all 3ds Max classes starting &apos;path&apos;\n\
showClass &quot;noise.*&quot; -- all the accessible properties on the\n\
-- Noise texture map\n\
showClass &quot;*:mod*&quot; -- all the modifier classes\n\
showClass &quot;*.*rad*&quot; -- all the classes with a property name\n\
-- containing &apos;rad&apos;\n\
showClass &quot;*.*&quot; to:f -- everything, out to a file\n\
showClass &quot;*:*controller*&quot; -- all the classes that have\n\
-- &quot;controller&quot; in their superclass\n\
-- name\n\
&gt;   showClass &quot;[X-Z]*&quot; -- all classes that start with X, Y, or Z\n\
</code></pre><p>In the output from the last example, the controllers include <strong>3ds Max</strong>&apos;s core controllers and any 3rd-party plug-in controllers. In some cases, embedded controllers for some complex plug-ins might be visible in the <code>showClass()</code> listing (such as for Character Studio Bipeds). You must <strong>not</strong> attempt to create and use these controllers individually, this might result in system exceptions.</p>\n\
<p>If you leave out the superclass pattern (the <code>&apos;:&apos;</code> part), it matches <strong>all</strong> superclasses. If you leave out the property pattern (the <code>&apos;.&apos;</code> part), you only get class names printed.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This function <strong>only</strong> displays matching MAXWrapper classes (nodes, modifiers, materials, and others), not the foundation classes in MAXScript (float, integer, array, point3, and others).</div>\n\
<p>The following shows examples of the <code>showClass()</code> output:</p>\n\
<p><strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showclass &quot;box*&quot; -- show all classes whose name starts with &quot;box&quot;\n\
showclass &quot;box.*&quot; -- show all properties for class box\n\
</code></pre><p><strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">Box : GeometryClass {10,0} -- start of output from line 1\n\
BoxGizmo : helper {3bc31904,67d74ec9}\n\
OK -- result returned from line 1\n\
Box : GeometryClass {10,0} -- start of output from line 2\n\
.height : float\n\
.length : float\n\
.lengthsegs : integer\n\
.width : float\n\
.widthsegs : integer\n\
.mapCoords : boolean\n\
.heightsegs : integer\n\
OK -- result returned from line 2\n\
</code></pre><p><a name=\"getobjectname\"></a></p>\n\
<h2 id=\"getobjectname\">getObjectName</h2><p>The getObjectName function returns the class name of an object, as it would appear in the history browser or modifier stack. If the passed object is not an object or modifier, this function returns undefined. The function has the following syntax:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getObjectName &lt;object&gt;&#x202F;[localizedName:&lt;bool&gt;] </code></pre><p><img src=\"./images/GUID-21C87158-BBA7-4B63-B8F7-CB9BA17CA2CA-low.gif\" alt> <a href=\"What-s-New-in-MAXScript-in-3ds.html\">NEW</a> <strong>in 3ds Max 2022:</strong> When the <code>localizedName</code> keyword parameter is <code>true</code> (the default), the localized name as it appears in the UI is returned.  Otherwise the non-localized name is returned.</p>\n\
<p>For example:</p>\n\
<p><strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">myBox = Box name:&quot;myBox&quot;\n\
myBend = Bend()\n\
myCompass = Compass()\n\
getObjectName myBox\n\
--&gt; &quot;Box&quot;\n\
getObjectName myBend\n\
--&gt; &quot;Bend&quot;\n\
getObjectName myCompass\n\
--&gt; &quot;Compass&quot;\n\
</code></pre><h2 id=\"showproperties\">showProperties</h2><p>The <code>showProperties()</code> function is used to display the properties for a specific object that is an instance of one of the MAXWrapper classes. It can be used in place of <code>showClass()</code> in situations where you have an actual object in hand. Unlike <code>showClass()</code> , it can display the <a href=\"Dynamic-Properties.html\">dynamic properties</a> that might appear in individual objects as you work on them in the scene such as, the sub-controllers in a list controller or the animated control points in an FFD modifier.</p>\n\
<p>The <code>showProperties()</code> function has the following form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showProperties &lt;maxwrapper_object&gt; [ &lt;property_pattern&gt; ] [ to :&lt;stream&gt; ]</code></pre><p>where <code>&lt;maxwrapper_object&gt;</code> is the <strong>3ds Max</strong> entity to be inspected, the optional <code>&lt;property_pattern&gt;</code> is a wild-card pattern that names the properties to be inspected, and the optional <code>to:&lt;stream&gt;</code> keyword argument specifies a <a href=\"./../../MAXScript-Language-Reference/Values/Stream-Values/Stream-Values-1.html\">Stream Value</a> to output the display to.</p>\n\
<p>The <code>&lt;property_pattern&gt;</code> is a pattern string.  See <a href=\"#pattern-strings\">Pattern Strings</a> above for more information about special characters in pattern strings.</p>\n\
<p>If the property name pattern is not supplied, <strong>all</strong> properties are listed.</p>\n\
<p><strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showProperties $foo.bend -- show properties of the Bend modifier on object foo \n\
ffdmod = $baz.&apos;FFD_box__4x4x4&apos; -- point to the FFD modifier on object baz\n\
showProperties ffdmod &quot;disp*&quot; to:log -- show properties starting with &quot;disp&quot; in the FFD modifier\n\
showProperties $foo.pos.controller -- sub controllers in a position list controller\n\
&gt;   showProperties $foo &quot;[t-z]*&quot; -- show properties starting with t to z on object foo\n\
</code></pre><div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>When the <strong>3ds Max</strong> entity specified is a node (for example, $box01), this function <strong>only</strong> displays the properties of the base object. It does not show the properties of the transform controllers, modifiers, or materials applied to the object. To display the properties for one of these objects, it must be specified as the <strong>3ds Max</strong> entity as shown in the above examples.</p>\n\
<p><strong>showProperties</strong> includes properties defined by FPS interfaces in <strong>3ds Max 8 and higher</strong>.</p>\n\
<p>The following shows examples of the <strong>showProperties</strong> output:</p>\n\
<p><strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">b = box() -- create a box\n\
ffd_mod = ffdBox() -- create a ffdBox modifier\n\
addmodifier b ffd_mod -- apply ffdBox modifier to the box\n\
showproperties b -- show all properties for the box\n\
showproperties ffd_mod -- show all properties for the ffdBox modifier \n\
</code></pre><p><strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">$Box:Box07 @ [0.0000,0.0000,0.0000] -- result from line 1\n\
FFD_box__4x4x4:FFD(box) 4x4x4 -- result from line 2\n\
OK -- result from line 3\n\
.height : float -- start of output from line 4\n\
.length : float\n\
.lengthsegs : integer\n\
.width : float\n\
.widthsegs : integer\n\
.mapCoords : boolean\n\
.heightsegs : integer\n\
OK -- result from line 4\n\
.dispLattice (Lattice) : boolean -- start of output from line 5\n\
.dispSource (Source Volume) : boolean\n\
.deformType (&lt;unknown&gt;) : integer\n\
.falloff : float\n\
.tension : float\n\
.continuity : float\n\
.inPoints (Inside Points) : boolean\n\
.outPoints (Outside Points) : boolean\n\
.offset : float\n\
.lattice_transform : transform\n\
OK -- result from line 5\n\
</code></pre><p>To make scripting easier to teach and use everyday, a ScriptTool function for &quot;Show&quot; has been added.</p>\n\
<p><strong>Show</strong> is the short way of typing <strong>ShowProperties</strong></p>\n\
<p><strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">Show $ --will show you the properties of the selected object\n\
</code></pre><h2 id=\"getpropnames\">getPropNames</h2><p>The <code>getPropNames()</code> function is similar to <code>showProperties()</code> , except it returns the property names for a specific object as an array. Unlike <code>showProperties()</code> , you can specify whether to return only the names of <a href=\"Dynamic-Properties.html\">dynamic properties</a> of the object.</p>\n\
<p>The <code>getPropNames</code> function has the form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropNames &lt;maxwrapper_obj&gt; [ #dynamicOnly]</code></pre><p>The <code>getPropNames()</code> function returns an array of the property names accessible on the object.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<p>if a FFD(Box) modifier with animated control points is applied to a sphere,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropNames $sphere01.&apos;FFD(box) 4x4x4&apos;\n\
</code></pre><p>could yield:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#(#dispLattice, #dispSource, #deformType, #falloff, #tension, #continuity, #inPoints, #outPoints, #offset, #Lattice_Transform, #Control_Point_49, #Control_Point_50, #Control_Point_51, #Control_Point_52, #Control_Point_53)\n\
</code></pre><div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>When the <strong>3ds Max</strong> entity specified is a node (for example, $box01), this function <strong>only</strong> returns the properties of the base object. It does not return the properties of the transform controllers, modifiers, or materials applied to the object. To return the properties for one of these objects, it must be specified as the <strong>3ds Max</strong> entity as shown in the above examples.</p>\n\
<p><strong>getPropNames</strong> includes properties defined by FPS interfaces in <strong>3ds Max 8 and higher</strong>.</p>\n\
<p>If you call <code>getPropNames()</code> on MAXWrapper classes such as, Box, Line, or Bend, it will return the properties common to all instances of that class. If you call <code>getPropNames()</code> on a <strong>3ds Max</strong> entity, it will return the properties currently defined for that entity. If you add the optional keyword <code>#dynamicOnly</code> to the <code>getPropNames()</code> call for an entity, it returns the dynamic properties unique to that instance.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<p>if a FFD(Box) modifier with animated control points is applied to a sphere,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropNames $sphere01.&apos;FFD(box) 4x4x4&apos; #dynamicOnly\n\
</code></pre><p>could yield:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#(#Lattice_Transform, #Control_Point_49, #Control_Point_50, #Control_Point_51, #Control_Point_52, #Control_Point_53)\n\
</code></pre><div class=\"note note:\"><span class=\"notetitle\">Note:</span> The property names for the lattice transform and control points which was returned by the previous example is not returned when <code>#dynamicOnly</code> is specified. These properties are unique to the specific FFD modifier.</div>\n\
<p>As a special case, you can also call <code>getPropNames()</code> on the superclass &apos;Node&apos; to get the list of properties common to all scene nodes such as, <code>.position</code> , <code>.name</code> , or <code>.wireColor</code> .</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropNames node\n\
</code></pre><h2 id=\"getpropnames-and-interfaces\">getPropNames and Interfaces</h2><p>The <code>getPropNames</code> method has been extended to work with the various interface values. The method will return the names of properties exposed by the interface.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getpropnames s.inode\n\
#(#boneEnable, #posTaskWeight, #rotTaskWeight, #boneAutoAlign, #boneFreezeLength, #boneScaleType, #stretchTM, #boneAxis, #boneAxisFlip, #primaryVisibility, #secondaryVisibility, #applyAtmospherics, #vertexColorType, #showVertexColors, #shadeVertexColors, #handle, #posInParent, #rotInParent, #scaleInParent, #scaleOrientInParent)\n\
</code></pre><p><a name=\"getproperty-and-setproperty\"></a></p>\n\
<h2 id=\"getproperty-and-setproperty\">getProperty and setProperty</h2><p>The <code>getProperty()</code> and <code>setProperty()</code> functions allow you to access and set properties using computed property names.</p>\n\
<p>The forms for each of these functions are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getProperty &lt;obj&gt; &lt;property_name&gt; [removeUIScaling:&lt;boolean&gt;]\n\
setProperty &lt;obj&gt; &lt;property_name&gt; &lt;value&gt; [applyUIScalling:&lt;boolean&gt;]</code></pre><p>The property name can be either a name, for example, <code>#length</code> , or a string, for example, <code>&quot;length&quot;</code> . You can construct property names with string operations.</p>\n\
<p>The <code>removeUIScaling</code>/<code>applyUIScaling</code> parameters default to true. If false, the value will be affected by the scale factor on High DPI displays, for those properties related to screen values (for example, #pos for UI controls). Available in <strong>in 3ds Max 2017 and higher</strong>.</p>\n\
<p><strong>EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getProperty foo #x -- equiv. to foo.x\n\
setProperty $foo &quot;radius&quot; 23 -- equiv. to $foo.radius = 23\n\
getProperty $foo.ffd_2x2x2 (&quot;control_point_&quot; + n as string)\n\
</code></pre><p>The <code>getPropNames()</code> function can be used in conjunction with the <code>getProperty()</code> function to implement object &apos;dumping&#x2019; tools that can iterate over all the properties of an arbitrary object and output them as required.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for p in getPropNames $foo do\n\
format &quot;% = %\\n&quot; p (getProperty $foo p)\n\
</code></pre><p>Similar functions for accessing and setting the values of property-assigned controllers are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropertyController &lt;value&gt; &lt;string_or_name&gt;\n\
setPropertyController &lt;value&gt; &lt;string_or_name&gt; &lt;controller&gt;</code></pre><p>Get and set the controller assigned to the named property of the value. If no controller has been assigned to the property, a value of undefined is returned.</p>\n\
<h2 id=\"hasproperty\">hasProperty</h2><p>The <code>hasProperty()</code> function allows you to test whether an object has a specified property name. Unlike <code>isProperty(),</code> the property name string supports wildcard patterns.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hasProperty &lt;obj&gt; &lt;prop_name_or_pattern_string&gt;</code></pre><p>Returns <code>true</code> if the object has the given property or has properties matching the string pattern.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hasProperty $Box01 &quot;height&quot; --test for height in a Box\n\
--&gt; true\n\
hasProperty $Box01 &quot;radius&quot; --test for radius in a Box\n\
--&gt; false\n\
hasProperty $Sphere01 &quot;radius&quot; --test for radius in a Sphere\n\
--&gt; true\n\
hasProperty $Sphere01 &quot;rad*&quot; --test for any property name\n\
--&gt; true --starting with &quot;rad&quot; in a Sphere\n\
</code></pre><h2 id=\"isproperty\">isProperty</h2><p>The <code>isProperty()</code> function allows you to test whether an object has a specified property.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">isProperty &lt;MAXWrapper_object&gt; &lt;property_name_or_string&gt;</code></pre><p><code>isProperty()</code> returns <code>true</code> if the specified property is a property of the MAXWrapper object, <code>false</code> otherwise. The property name can be either a name, for example, <code>#length</code> , or a string, for example, <code>&quot;length&quot;</code> . You can construct property names with string operations.</p>\n\
<p><strong>EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">b=box()\n\
s=sphere()\n\
isProperty $Box001 #height --test for height in a Box\n\
--&gt; true\n\
isProperty $Box001 #radius --test for radius in a Box\n\
--&gt; false\n\
isProperty $Sphere001 #radius --test for radius in a Sphere\n\
--&gt; true\n\
</code></pre><h2 id=\"the-difference-between-hasproperty-and-isproperty\">The difference between hasProperty and isProperty</h2><p><strong>hasProperty</strong> and <strong>isProperty</strong> do entirely different things.</p>\n\
<p><strong>EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hasProperty $Box01 &quot;name&quot; \n\
--&gt; false\n\
isProperty $Box01 &quot;name&quot; \n\
--&gt; true\n\
</code></pre><p><strong>hasProperty</strong> uses the same code as <code>showPropeties()</code> , but instead of returning a list of the properties found, it returns true on the first property name that matches the specified pattern. <strong>showProperties</strong> is looking at just the box base object and its MAXClass definition. It does not look at any higher level properties (and in versions prior to <strong>3ds Max 8</strong> it does not look at any of the FPS interface exposed properties).</p>\n\
<p><strong>isProperty</strong> on the other hand actually tries to access the specified object with the specified name and to detect whether the access succeeded or failed.</p>\n\
<h2 id=\"ispropertyanimatable\">isPropertyAnimatable</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">isPropertyAnimatable &lt;obj&gt; &lt;string_or_name&gt;</code></pre><p>Returns <code>true</code> if the property is found and is animatable. Returns <code>false</code> if the property is not found or is not animatable.</p>\n\
<p><strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">s = Sphere() --create a sphere\n\
--&gt; $Sphere:Sphere01 @ [0.000000,0.000000,0.000000]\n\
isPropertyAnimatable s #radius --check if radius is animatable\n\
--&gt; true\n\
isPropertyAnimatable s #mapCoords --check if UV coords are\n\
--&gt; false\n\
</code></pre><p><strong>ANOTHER EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">s = Sphere() --create a sphere\n\
--Print a list of all properties of the sphere and whether\n\
--they are animatable or not:\n\
for i in getPropNames s do\n\
format &quot;% Property: % - Animatable: %\\n&quot;s.name i (isPropertyAnimatable s i)\n\
</code></pre><p><strong>RESULT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">$Sphere:Sphere003 @ [0.000000,0.000000,0.000000]\n\
Sphere003 Property: #smooth - Animatable: true\n\
Sphere003 Property: #radius - Animatable: true\n\
Sphere003 Property: #segs - Animatable: true\n\
Sphere003 Property: #mapcoords - Animatable: false\n\
Sphere003 Property: #slice - Animatable: false\n\
Sphere003 Property: #hemisphere - Animatable: true\n\
Sphere003 Property: #sliceFrom - Animatable: true\n\
Sphere003 Property: #sliceTo - Animatable: true\n\
Sphere003 Property: #chop - Animatable: false\n\
Sphere003 Property: #recenter - Animatable: false\n\
Sphere003 Property: #realWorldMapSize - Animatable: false\n\
OK\n\
</code></pre><h2 id=\"subanims-and-paramblocks\">SubAnims and ParamBlocks</h2><p><strong>3ds Max</strong> stores all of properties for objects in structures called <em>subAnims</em>. In some cases, a subAnim will contain nested subAnims. An example of this can be seen with the Standard material. If you apply a Standard material to a <strong>3ds Max</strong> object, and then expand the object&apos;s tracks in Track View to display the material, you will see three Track View nodes - Parameters, Maps, and Shader. Each of these nodes is a subAnim defined in the Standard material&#x2019;s class. Expanding the tracks for these three nodes, you will see various properties listed. These properties are defined in each of the nested subAnims in a structure called a <em>ParamBlock</em>.</p>\n\
<p>MAXScript automatically finds animatable properties in all ParamBlocks associated with an object. To access these properties, you use the same nesting as shown in Track View when referencing those properties in MAXScript.</p>\n\
<p><strong>FOR EXAMPLE,</strong></p>\n\
<p>you would access the Ambient Color property in a Standard material in MAXScript as:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">$box01.material.shader.diffuse_color\n\
</code></pre><p>For more information about accessing animatable properties in MAXScript, see also <a href=\"Indexed-Access-to-Animatable.html\">Indexed Access to Animatable Properties in 3ds Max Objects</a>.</p>\n\
<p>The properties accessible on various kinds of values are documented for each class. Many of the foundation classes in MAXScript (such as, point3, color, and so on) support a special kind of <strong>nested-property assignment</strong> which allows sub-properties on object properties such as, the <code>.x</code> , <code>.y</code> , and <code>.z</code> coordinates in a node&apos;s <code>.position</code> property to be individually and directly set. See <a href=\"./../MAXWrapper-Value/Nested-Object-Properties.html\">Nested Object Properties</a> for details.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Identifying-and-Accessing.html\">Identifying and Accessing MAXScript Classes and Properties</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Interfaces/showInterfaces-Inspector.html\">showInterfaces Inspector Function</a></li>\n\
          <li><a class=\"link\" href=\"../Interfaces/GetInterfaces-Interface.html\">GetInterfaces Interface Inspector</a></li>\n\
          <li><a class=\"link\" href=\"../Interfaces/showInterface-Inspector-Function.html\">showInterface Inspector Function</a></li>\n\
          <li><a class=\"link\" href=\"getClassInstances.html\">getClassInstances</a></li>\n\
          <li><a class=\"link\" href=\"../Interfaces/Core-Interfaces/Core-Interfaces-Documentation.html\">Core Interfaces Documentation</a></li>\n\
          <li><a class=\"link\" href=\"../Interfaces/Other-Interfaces/Other-Interfaces-1.html\">Other Interfaces</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";