var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-AF1F51D4-449B-4C4D-9F58-85DB145BC0B9\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Node User-Defined Properties and Methods</h1></div><p><img src=\"./images/GUID-02ADD04B-DC3F-4E59-A48B-EAD697B25BBE-low.gif\" alt></p>\n\
<p>The functions described in this topic give you access to scene object user properties, accessible in the <strong>3ds Max</strong> user interface in the <strong>User Defined</strong> tab in the <strong>Object Properties Dialog</strong>.</p>\n\
<p>There are two sets of MAXScript methods for setting and getting individual keyed properties, and two methods that let you treat the property text as a single big string. The user defined properties in the Object Properties dialog are accessed as one single string representing the entire contents or as key/property pairs in the form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;key1&gt; = &lt;property1&gt;&lt;key2&gt; = &lt;property2&gt;...</code></pre><p>where, the keys are identifiers (Name or String values, though Names are not recommended, see the note below) and the properties can be values. For the old methods, these can be numbers, booleans, or text strings. For the new methods, any value where <code>(val==execute(val as string))</code> is true can be used.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The <code>setUserProp()</code>/<code>getUserProp()</code> methods do not always produce predictable results. The <code>setUserPropVal()</code>/<code>getUserPropVal()</code> methods are the preferred way to get/set user properties, as of <strong>3ds Max 2019.1 Update</strong>.</div>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span>  The key is case sensitive. If a name value is used, it is converted to the string originally used to define the name value, which might be different than the case of the specified name value itself. For this reason, we do not recommend you use name values as keys for any of these methods.</div>\n\
<p>The <code>&lt;node&gt;</code> methods are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getUserPropVal &lt;node&gt; &lt;key_string&gt; [asString:&lt;boolean&gt;] [decodeCRLF:&lt;boolean&gt;]</code></pre><p>Retrieves the node&apos;s user property, where the property key corresponds to <code>&lt;key_string&gt;</code>. Available in <strong>3ds Max 2019.1 Update and higher</strong>. The property value&apos;s string is evaluated as a MaxScript value if <strong>asString</strong> is not specified, or is specified as false. If the evaluation of the value&apos;s string is successful, that value is returned, otherwise the value&apos;s string is returned as a string value. If <strong>asString</strong> is specified as true, the property value&apos;s string is always returned as a string value.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which gets a user defined property for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong></p>\n\
<p>When returning a string value, if the first and last character of the string are quotes, those quotes are removed from the string. When returning a string value, if <strong>decodeCRLF</strong> is not specified or is specified as true, any carriage return or line feed hex representations in the property value&apos;s string are converted to carriage return or line feed characters, otherwise they are not. That is, &apos;\\xd\\xa&apos; would be converted to &apos;\\r\\n&apos;.</p>\n\
<p>If the property key does not exist, a value of undefined is returned. If the property key exists but no value is present, a value of &apos;emptyVal&apos; is returned.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> Using <code>getUserPropVal()</code> to retrieve properties written either in the Object Properties dialog or with <code>setUserProp()</code> may give unexpected results. For example:<code>setUserProp &quot;p1&quot; &quot;xxx&quot;; v=getUserPropVal &quot;p1&quot;</code>. In this case, v will most likely be &apos;undefined&apos;, because it was not saved in quotes, so is considered a value representation (a global variable name), and the contents of the variable name are returned. This is essentially the same as <code>v=execute &quot;xxx&quot;</code>.</div>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The special value type <code>emptyVal</code> (a distinguished instance of the <code>Empty</code> class) is used to indicate properties that have a key but no value. For example, if the property key &quot;xxx&quot; is defined without a value, calling <code>getUserPropVal()</code> returns an &apos;emptyVal&apos;. For keys that are not defined, the same call returns &apos;undefined&apos;.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setUserPropVal &lt;node&gt; &lt;key_string&gt; &lt;value&gt;  [quoteString:&lt;boolean&gt;] </code></pre><p>Sets a user property on node, where the property key corresponds to <code>&lt;key_string&gt;</code> and the property value is corresponds to <code>&lt;value&gt;</code>. Available in <strong>3ds Max 2019.1 Update and higher</strong>. Any value type where <code>(val == execute (val as string))</code> can be used. If <code>&lt;value&gt;</code> is a string, and <strong>quoteStrings</strong> is not specified or is specified as true, the property value&apos;s string is enclosed in quotes, otherwise it is not. If <code>&lt;value&gt;</code> is a string, and <strong>encodeCRLF</strong> is not specified or is specified as true, any carriage return or line feed characters in the property value&apos;s string are converted to hex representations, otherwise they are not. That is, &apos;\\r\\n&apos; would be converted to &apos;\\xd\\xa&apos;. If a value of &apos;emptyVal&apos; is used, the key is written but no value is written.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which sets a user defined property for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getUserProp &lt;node&gt; &lt;key_string&gt;      </code></pre><p>Retrieves the node&apos;s user property, where the property key corresponds to <code>&lt;key_string&gt;</code>. If the key does not exist, a value of <code>undefined</code> is returned. MAXScript tries to convert the property value to a number, if that fails then to a boolean, and if that fails it returns the value as a text string. This can cause incorrect conversions in some cases. For example, if the property value was set to &quot;1m&quot;, <code>getUserProp()</code> returns a Time value of 1800f since &quot;1m&quot; is interpreted as a Time literal of 1 minute. For this reason you should use <strong>getUserPropVal()</strong> instead.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which gets the user defined properties for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setUserProp &lt;node&gt; &lt;key_string&gt; &lt;value&gt;   </code></pre><p>Sets the node&apos;s user property with the given key to the string representation of the given value.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which sets a user defined property buffer for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;doesUserPropExist &lt;node&gt; &lt;key_string&gt;</code></pre><p>Returns true if the specified user property exists on the node. Available in <strong>3ds Max 2019.1 Update and higher</strong>.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which looks for user defined property for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;deleteUserProp &lt;node&gt; &lt;key_string&gt; </code></pre><p>Deletes specified user property on node. Available in <strong>3ds Max 2019.1 Update and higher</strong>. Returns true if the property existed.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which deletes a user defined property for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getUserPropBuffer &lt;node&gt; </code></pre><p>Retrieves the entire user property buffer as a string containing all the user property settings. This is effectively the contents of the User Defined Properties box in the <strong>3ds Max</strong> Object Properties dialog.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which gets the user defined properties for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setUserPropBuffer &lt;node&gt; &lt;string&gt; </code></pre><p>Sets the user property buffer to the given string.</p>\n\
<p>The node parameter can be specified as <code>rootNode</code> or an XRef root node (the <code>.tree</code> property), which sets a user defined property buffer for that root node. Available in <strong>3ds Max 2019.2 Update and higher</strong>.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>To include new line characters in a multi-line User Property Buffer string, you have to use the combination <strong>\\r\\n</strong> (carriage return and new line). See <a href=\"./../../../MAXScript-Language-Reference/Names-Literal-Constants-and/Literal-Constants/String-Literals.html\">String Literals</a> for a list of control characters.</p>\n\
<p>The following two example code fragments save and load user properties for objects in a scene.</p>\n\
<p>The first code fragment creates a file and loops through all objects outputting for each the object name and user property buffer string. Note that the object name goes out in a form that will come back in (through a <code>readValue()</code> ) as a direct reference to a scene object. Also, the user property string is output using <code>print()</code> , so it will be in a quoted form to be read as one (long) string literal by a corresponding <code>readValue()</code> .</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   -- create file and for all objects,\n\
   -- dump object name and user props\n\
   f = createFile &quot;foo.dat&quot;\n\
   for o in objects do\n\
   (\n\
   format &quot;$%\\n&quot; o.name to:f -- output name in a pathname form\n\
   print (getUserPropBuffer o) to:f -- output buffer as a string literal\n\
   )\n\
   close f\n\
</code></pre></blockquote>\n\
<p>The second code fragment reads in the file created by the first piece and applies the user properties to any same-named object in the current scene. Note that <code>readValue()</code> can read in pathnames ( <code>$&lt;name&gt;</code> ) and will yield either the named scene object or <code>undefined</code> if the named object is not in the scene. The <code>readValue()</code> function can also read in a multiline string literal in one pass.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   -- open file and read in object names and\n\
   -- user properties to set\n\
   f = openFile &quot;foo.dat&quot;\n\
   while not eof f do\n\
   (\n\
   o = readValue f-- read object\n\
   if o != undefined then-- if present, read and set user prop string\n\
   setUserPropBuffer o (readValue f)\n\
   )\n\
   close f\n\
</code></pre></blockquote>\n\
<p>You can access the user property buffer on an XRef by first getting the XRef with <code>xrefs.getXRefFile()</code>, and then getting the XRef root node with the <code>.tree</code> property.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   -- Properties on an xref scene\n\
   i = 1  -- index value\n\
   z = xrefs.getXRefFile i\n\
   getUserPropBuffer (z.tree)</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Node-Common-Properties-Operators.html\">Node Common Properties, Operators, and Methods</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";