var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-279879A8-9844-4C29-9EA8-43E5FF922692\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>For Loop By Index Vs For Loop Through Collection Performance</h1></div><h2 id=\"a-user-asked-\">A user asked:</h2><p>What is the difference between using</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for i = 1 to theArray.count do( ... )</code></pre><p>and</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for i in theArray do( ... )</code></pre><p>Are they the same? Is one faster than the other ? Anything else to be aware of?</p>\n\
<h2 id=\"answer-\">Answer:</h2><p>Both can be used to do the same, but have different strengths and performance characteristics.</p>\n\
<p>The first version gives you an index. So, if you want to drive a progress bar display, or use the index to access another array with the same amount of elements, it is usually the one to use - you can still get the i-th object, but you also have the number of objects that you have processed so far for free.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   theArray = for i = 1 to 10  collect i\n\
   (\n\
   for i = 1 to theArray.count do\n\
   (\n\
     --assuming prg_bar is a progressbar UI control:\n\
     --prg_bar.value = 100.0*i/theArray.count\n\
     --some more code here...\n\
   )\n\
   )\n\
</code></pre></blockquote>\n\
<p>In the second case, you have to add your own counter local variable to keep track of the current indexes:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   cnt = 0\n\
   for i in theArray do\n\
   (\n\
     cnt+=1\n\
     --prg_bar.value = 100.0*cnt/theArray.count\n\
     --some more code...\n\
   )\n\
   )\n\
</code></pre></blockquote>\n\
<p>Also, the first form allows you to loop backwards, which is essential if you are removing elements from the array while processing it. The following example creates an array of 1000 random integers and then in the second for loop it deletes any elements with a value less than 50:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   theArray =for i = 1 to 1000 collect random 1 100\n\
   for i = theArray.count to 1 by -1 where theArray[i] &lt; 50 do\n\
     deleteItem theArray i\n\
   )\n\
</code></pre></blockquote>\n\
<p>When it comes to speed though, the direct loop through the array elements is faster. Let us create an array of 10000 elements and loop through it 1000 times to get a significant time value to measure. The inner loop itself does not perform any actual work, only loops through the array and accesses the i-th element.</p>\n\
<blockquote>\n\
<p>   <strong>LOOP BY INDEX:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   theArray = for i = 1 to 10000 collect random 1 100\n\
   st = timestamp()\n\
   for j = 1 to 1000 do\n\
     for i = 1 to theArray.count do theArray[i]\n\
   format &quot;% ms\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>This code executes on a particular machine in 5.6 seconds (+/- 0.1 seconds).</p>\n\
<p>Removing the actual access to the array element by index (theArray[i]) reduces this time to 4.6 seconds. For example, if you were running from 1 to the array count, but not using the array elements for anything, which is highly improbable.</p>\n\
<blockquote>\n\
<p>   <strong>IN COMPARISON, LOOPING THROUGH THE COLLECTION</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   theArray =for i = 1 to 10000 collect random 1 100\n\
   st = timestamp()\n\
   for j = 1 to 1000 do\n\
     for i in theArray do i\n\
       format &quot;% ms\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>This executes on the same machine in 4.3 seconds (+/- 0.1 sec.), which is 1.3 times faster. Because the variable i already contains the n-th element, it does not matter whether we actually use the array element value for anything or not.</p>\n\
<p>Now, if you do need the counter for something else like progress updates, the first approach is almost four times faster than the second.</p>\n\
<blockquote>\n\
<p>   <strong>SLOW</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   theArray =for i = 1 to 10000 collect random 1 100\n\
   st = timestamp()\n\
   for j = 1 to 1000 do\n\
   (\n\
     cnt = 0\n\
     for i in theArray do cnt +=1\n\
   )\n\
   format &quot;% ms\\n&quot; (timestamp()-st) \n\
   )\n\
</code></pre></blockquote>\n\
<p>The above code executes in 14.3 seconds on the same machine because of the memory overhead involved in overwriting the value of the variable cnt 10 million times.</p>\n\
<p>In other words, if the counter variable is significant for your code, you can use the form <strong>for i = 1 to theArray.count do()</strong>, otherwise use the direct loop through the collection as it is slightly faster in the general case.</p>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"Try-not-to-use-Execute-function.html\">Try not to use Execute function if there is an alternative</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";