var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-F6D660D1-D0EE-41DB-804D-7404E513E2B6\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>For Loop Collect Vs For Loop Do Performance</h1></div><h2 id=\"a-user-asked-\">A user asked:</h2><p>I was quickly making a script to select all objects and I realized that it was very slow, even with undo off, disableSceneRedraw and suspendEditing.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for o in objects do selectmore o</code></pre><p>I thought a bit and discovered that I could select an array of objects like in this code:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">a=#()\n\
for o in objects do append a o\n\
selectmore a</code></pre><p>The difference in speed is really huge (like 3000%+ faster).</p>\n\
<p>But there is absolutely no reference to the possibility of selecting an array of objects in the MAXScript help, even in the &quot;how to make it faster&quot; section.</p>\n\
<p>Is there anything I should know about that way of selecting objects?</p>\n\
<p>Why isn&apos;t that documented a tall as it looks very effective?</p>\n\
<h2 id=\"answer-\">Answer:</h2><p>The truth is people tend to code differently.</p>\n\
<p>All the info needed was in the Reference, but it did not really tell you &quot;Don&apos;t do this&quot; because nobody can anticipate every possible expression someone might write.</p>\n\
<p>Doing things one at a time instead of once for the whole collection sounds like it would be slow, and it is.</p>\n\
<p>The first example loops through an ObjectSet and sets the isSelected property of the node to true for each one, which is very very slow.</p>\n\
<p>The second example does indeed call selectMore() in a mapped fashion, but the array building is a bit inefficient, as it uses append() which has to grow the array on each call by duplicating the previous array in memory. With few objects it would be negligible though, with a lot more objects it might start causing slowdowns...</p>\n\
<blockquote>\n\
<p>   <strong>POSSIBLE EXPRESSIONS</strong></p>\n\
<p>   If you really have to do a FOR loop, use the COLLECT form instead of DO. It builds the array and gives back the result without using append() explicitly or any intermediate variables to store the array. You could assign to a vatiable if you want to keep the selection snapshot around.</p>\n\
<p>   It would be something like:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   select (for o in objects collect o)\n\
</code></pre><p>   This would allow you to apply where conditions, for example to select all visible objects,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   select (for o in objects where not o.isHiddenInVpt collect o)\n\
</code></pre><p>   or, if this really is what you are trying to do, selecting ALL scene object, simply</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   select objects\n\
</code></pre></blockquote>\n\
<p>There is no reason to do selectmore() if you are selecting all scene objects anyway, and you can operate on the live collection of scene objects without converting it to array. Note that select() will make sure the previous selection is removed, while selectMore() will obviously add to the existing one.</p>\n\
<blockquote>\n\
<p>   <strong>SOME BENCHMARK RESULTS</strong></p>\n\
<p>   The time is measured on the same reference computer around the select() method and does NOT include the time it takes the viewport(s) to actually redraw, just the time to set the selected flags of the nodes.</p>\n\
<p>   <strong>EXPRESSION 1 - SELECT ALL</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   select objects\n\
</code></pre><p>   With 100objects (boxinstances), the above expressiontook31 ms.</p>\n\
<p>   With 1,000objects, it took 78ms.</p>\n\
<p>   With 10,000objects, it took 328ms.</p>\n\
<p>   With 100,000objects, it took 15,468ms.</p>\n\
<p>   <strong>EXPRESSION 2 - DO, APPEND AND SELECT</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   a=#()\n\
   for o in objects do append a o\n\
   select a\n\
</code></pre><p>   With 100, 1000 and 10,000 objects, the above expression performed similarly to the first one, proving that the memory overhead of the array management is not measurable with relatively low numbers of iterations.</p>\n\
<p>   The code is longer and less elegant though.</p>\n\
<p>   With 100,000 objects, the expression took 17,547ms., or about two seconds more than the first expression.</p>\n\
<p>   <strong>EXPRESSION 3 - TEST, COLLECT AND SELECT</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   select (for o in objects where not o.isHiddenInVpt collect o)\n\
</code></pre><p>   The above expression took around 430ms with 1000 objects.</p>\n\
<p>   Performing a condition test on 1000 objects does not make a significant difference, but increases the flexibility of the expression to select specific objects by their properties.</p>\n\
<p>   <strong>EXPRESSION 4 - DO AND SELECT MORE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for o in objects do selectMore o\n\
</code></pre><p>   The above expression took only <strong>171 ms</strong> if all 1000 objects were <strong>already selected</strong> because it had nothing to do, but <strong>674172 ms</strong> (over <strong>11 minutes!!!</strong>) if none of the 1000 objects were selected!</p>\n\
<p>   <strong>STAY AWAY FROM IT!</strong></p>\n\
</blockquote>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"For-Loop-By-Index-Vs-For-Loop.html\">For Loop By Index Vs For Loop Through Collection Performance</a></p>\n\
<p><strong>Next Tip</strong></p>\n\
<p><a href=\"Optimizing-Particle-Flow-Script.html\">Optimizing Particle Flow Script Operator For Loops For Speed</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";