var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-C94469C6-19A5-427E-A16B-D46054B3AA1A\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How To ... Develop A Face Area XView Checker - Part 1</h1></div><p>The following tutorial demonstrates the creation of a <strong>custom xView Checker</strong> using MAXScript functions.</p>\n\
<p>The xView Checker is available in <strong>3ds Max 2010</strong> and higher.</p>\n\
<p><strong>Related topics:</strong></p>\n\
<p><a href=\"./../../MAXScript-Tools-and-Interaction/Interacting-with-the-3ds-Max/Geometry-Checker/xView-Geometry-Checker-Access.html\">xViewChecker Access</a></p>\n\
<p><a href=\"./../../3ds-Max-Objects-and-Interfaces/Interfaces/Core-Interfaces/Core-Interfaces-Documentation/U-V-W-X-Y-Z/Interface-xViewChecker.html\">Interface:xViewChecker</a></p>\n\
<blockquote>\n\
<p>   <strong>NATURAL LANGUAGE</strong></p>\n\
<p>   Create a global struct with properties and functions</p>\n\
<p>   Define an xView geometry checker function</p>\n\
<p>   Define a supported function</p>\n\
<p>   Register xView Checker.</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   global FaceAreaChecker\n\
   struct FaceAreaCheckerStruct\n\
   (\n\
   FaceAreaThresholdMin = 1,\n\
   FaceAreaThresholdMax = 10,\n\
   faceAreaDialog = undefined,\n\
\n\
   fn geomCheck theTime theNode theResults =\n\
   (\n\
     local theCount = case classof theNode of\n\
     (\n\
       Editable_Poly: getNumFaces theNode\n\
       Editable_Mesh: theNode.numfaces\n\
     )\n\
     local theAreaMethod = case classof theNode of\n\
     (\n\
       Editable_Poly: polyOp.getFaceArea\n\
       Editable_Mesh: meshOp.getFaceArea\n\
     )\n\
\n\
     for f = 1 to theCount do\n\
     (\n\
       local theArea = theAreaMethod theNode f\n\
       if theArea &gt;= FaceAreaChecker.FaceAreaThresholdMin and theArea &lt;= FaceAreaChecker.FaceAreaThresholdMax do append theResults f\n\
     ) \n\
   3\n\
   ),\n\
\n\
   fn supportedCheck theNode =\n\
   (\n\
     classof theNode == Editable_Mesh or classof theNode == Editable_Poly\n\
   ),\n\
\n\
   fn configDlg =\n\
   (\n\
     try(destroyDialog FaceAreaChecker.faceAreaDialog)catch()\n\
     rollout faceAreaDialog &quot;Face Area Checker&quot;\n\
     (\n\
       spinner spn_FaceAreaThresholdMin &quot;Min. Threshold:&quot; range:[0,1000000,FaceAreaChecker.FaceAreaThresholdMin] offset:[7,-3] type:#worldUnits\n\
       spinner spn_FaceAreaThresholdMax &quot;Max. Threshold:&quot; range:[0,1000000,FaceAreaChecker.FaceAreaThresholdMax] offset:[7,-3] type:#worldUnits\n\
\n\
       fn updateDisplay =\n\
       (\n\
         XViewChecker.runCheck CurrentTime\n\
         max views redraw\n\
       )\n\
       on spn_FaceAreaThresholdMin changed val do\n\
       (\n\
         FaceAreaChecker.FaceAreaThresholdMin = val\n\
         updateDisplay()\n\
       ) \n\
       on spn_FaceAreaThresholdMax changed val do\n\
       (\n\
         FaceAreaChecker.FaceAreaThresholdMax = val\n\
         updateDisplay()\n\
       )   \n\
     )--end rollout\n\
\n\
     createDialog faceAreaDialog\n\
     setDialogPos faceAreaDialog mouse.screenpos\n\
     FaceAreaChecker.faceAreaDialog = faceAreaDialog\n\
   )\n\
   )--end struct\n\
\n\
   try(destroyDialog FaceAreaChecker.faceAreaDialog)catch()\n\
   FaceAreaChecker = FaceAreaCheckerStruct()\n\
\n\
   XViewChecker.unRegisterChecker &quot;Face Area Checker&quot;\n\
   XViewChecker.registerChecker FaceAreaChecker.geomCheck FaceAreaChecker.supportedCheck #Faces &quot;Face Area Checker&quot; FaceAreaChecker.configDlg undefined undefined\n\
   )--end script\n\
</code></pre></blockquote>\n\
<h2 id=\"step-by-step\">Step-By-Step</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">(</code></pre><p>The complete script is in a local scope.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">global FaceAreaChecker</code></pre><p>Only the instance of the struct containing the xView Checker properties and functions will be exposed to the global scope.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">struct FaceAreaCheckerStruct (</code></pre><p>We start a struct to encapsulate all properties and functions. It will be local to the current scope.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">FaceAreaThresholdMin = 1,</code></pre><p>This property will contain the minimum face area to check for.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">FaceAreaThresholdMax = 10,</code></pre><p>This property will contain the maximum face area to check for.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">faceAreaDialog = undefined,</code></pre><p>Our xView Checker will include a settings dialog. To be able to close it between sessions, we will store the rollout of the dialog in this property. This will give us global access to the dialog without directly storing it in a global variable.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn geomCheck theTime theNode theResults =\n\
(</code></pre><p>This is the main xViewChecker function.</p>\n\
<p>An xView Checker function must expect three arguments which are passed to it by the system - the time at which the check is performed, the node being checked, and an array to put the results from the check into.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local theCount = case classof theNode of\n\
(\n\
Editable_Poly: getNumFaces theNode\n\
Editable_Mesh: theNode.numfaces\n\
)</code></pre><p>Since we want to be able to check both Editable Poly objects and Editable Meshes, we check the class of the node passed to the function and get the face or polygon count using the correct method or property.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local theAreaMethod = case classof theNode of\n\
(\n\
Editable_Poly: polyOp.getFaceArea\n\
Editable_Mesh: meshOp.getFaceArea\n\
)</code></pre><p>Again, based on the class of the node, we store the method to get the polygon or face area in a user variable. This will both speed up the access to the method and allow us to use the same expression to get the face area in the following loop.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for f = 1 to theCount do\n\
(</code></pre><p>We loop through all the faces or polygons of the node</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local theArea = theAreaMethod theNode f</code></pre><p>We call the cached method for polygon resp. face area and pass the node and the face index to it.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">if theArea &gt;= FaceAreaChecker.FaceAreaThresholdMin and theArea &lt;= FaceAreaChecker.FaceAreaThresholdMax do append theResults f</code></pre><p>Then we can compare the value to the min. and max. threshold and if the area is between the two thresholds, we append the face index to the results array.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">) \n\
3</code></pre><p>The xViewCheck function must return an integer specifying the result type.</p>\n\
<p>We return 3 because we stored the indices of the faces in the results array.</p>\n\
<p>The other options are 2 for edges, 1 for vertices and 0 if the check failed.</p>\n\
<p>This will help the rest of the xView Checker system figure out how to use the data in the results array to display in the viewport or convert the results to a sub-object selection.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">),\n\
\n\
fn supportedCheck theNode =\n\
(\n\
classof theNode == Editable_Mesh or classof theNode == Editable_Poly\n\
),</code></pre><p>The supported check function is called by the xView Checker system to determine whether a node supports the Checker or not. In our case, we want to allow the checking of Editable Meshes and Editable Poly objects, so we check the class of the node and compare to these classes. Any other objects will be skipped.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn configDlg =\n\
(</code></pre><p>This function will be called by the xView Checker system whenever the user clicks on the &quot;Click Here To Configure&quot; button in the Viewport. If this function was not registered with the checker (but passed as undefined value instead), that viewport button would not be displayed and the system would assume the Checker does not support a configuration dialog.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">try (destroyDialog FaceAreaChecker.faceAreaDialog)catch()</code></pre><p>We attempt to close any dialog that might have been opened previously. We use the property of the struct instance where we will assign the rollout used by the dialog.</p>\n\
<p>If the dialog is not open, nothing will happen.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">rollout faceAreaDialog &quot;Face Area Checker&quot;\n\
(</code></pre><p>Now we define a rollout to display as a configuration dialog.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">spinner spn_FaceAreaThresholdMin &quot;Min. Threshold:&quot; range:[0,1000000,FaceAreaChecker.FaceAreaThresholdMin] offset:[7,-3] type:#worldUnits\n\
spinner spn_FaceAreaThresholdMax &quot;Max. Threshold:&quot; range:[0,1000000,FaceAreaChecker.FaceAreaThresholdMax] offset:[7,-3] type:#worldUnits </code></pre><p>We define two spinners to control the min. and max. threshold values used by the test. Both spinners are set to default to the current properties of the struct instance and display world units. Of course, we will be checking for SQUARE units, but the spinners will show just regular linear units. So if the spinner shows 10&quot;, we will interpret the value as 10 square inches.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn updateDisplay =\n\
(\n\
XViewChecker.runCheck CurrentTime\n\
max views redraw\n\
)</code></pre><p>This function will be called when User Interface elements (the spinners) change their values. The function calls the method to run the current xView check at the current time, resulting in updated checker data. To force the display of the new data, we have to do a full redraw of all viewports.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on spn_FaceAreaThresholdMin changed val do\n\
(\n\
FaceAreaChecker.FaceAreaThresholdMin = val\n\
updateDisplay()\n\
) </code></pre><p>Whenever the min. threshold spinner is changed, we store the new value in the struct instance&apos;s property and call the update function to refresh the viewports.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on spn_FaceAreaThresholdMax changed val do\n\
(\n\
FaceAreaChecker.FaceAreaThresholdMax = val\n\
updateDisplay()\n\
)   </code></pre><p>We do the same with the max. threshold spinner.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end rollout \n\
\n\
createDialog faceAreaDialog</code></pre><p>At this point, we can create a dialog from the rollout. The size of the dialog will be set automatically based on the size of the controls in the rollout.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setDialogPos faceAreaDialog mouse.screenpos</code></pre><p>Then we reposition the dialog to the mouse position, so when the user clicks the &quot;Click Here To Configure&quot; button in the viewport, the dialog will appear next to the mouse pointer.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">FaceAreaChecker.faceAreaDialog = faceAreaDialog</code></pre><p>Finally, we store the rollout definition in the property of the struct instance. This way, the attempt to close the dialog before the rollout definition will access the correct value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">) )--end struct\n\
\n\
try (destroyDialog FaceAreaChecker.faceAreaDialog)catch()</code></pre><p>This attempt to close the dialog is useful when developing the script and re-evaluating the source again and again - each time the script is run, a previous open dialog will be closed. Otherwise, orphaned dialogs could accumulate during testing.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">FaceAreaChecker = FaceAreaCheckerStruct()</code></pre><p>Here we create the instance of the local struct and assign to the global variable. This way, any other script or the Listener can access the properties and functions inside the struct instance, which simplifies both development and debugging.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">XViewChecker.unRegisterChecker &quot;Face Area Checker&quot;</code></pre><p>We unregister the XView Checker by name. If the same code is run multiple times, we ensure the previous version is removed before registering it again.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">XViewChecker.registerChecker FaceAreaChecker.geomCheck FaceAreaChecker.supportedCheck #Faces &quot;Face Area Checker&quot; FaceAreaChecker.configDlg undefined undefined</code></pre><p>Finally, we register our Checker with the xViewChecker system.</p>\n\
<p>The first argument is the function to call to check scene objects.</p>\n\
<p>The second argument is the function to call to determine if a scene object can be checked.</p>\n\
<p>The third argument defines the type of the checker - whether it operates on Faces, Edges or Vertices. This is necessary for the Selection conversion to interpret the content of the results array as the right type of Sub-Object indices.</p>\n\
<p>The fourth argument is the name of the checker. It is used both to display the checker in the list and the viewport and to access the checker by name when unregistering it.</p>\n\
<p>The fifth argument is the function to call when the &quot;Click Here To Configure&quot; button is clicked. It can be passed as undefined if no dialog is necessary.</p>\n\
<p>The sixth argument is the function to call to override the text in the viewport next to the Checker&apos;s name. We pass undefined since we don&apos;t want a text override at this point. We will implement such a function in the next part of the tutorial.</p>\n\
<p>The last argument is the function to call to override the default display functionality. We pass again undefined because we want to use the default display feature at this point. We will implement such a function in the next part of the tutorial.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end script</code></pre><h2 id=\"using-the-xview-checker\">Using the xView Checker</h2><p>At this point, we can evaluate the code by pressing <strong>Ctrl+E</strong> and use the xView checker, but it will not be visible in the Viewport Menus yet.</p>\n\
<p>To see the new checker, enable any other xView checker like for example <strong>Face Orientation</strong>, then click on the [Face Orientation] label at the bottom of the viewport to display the list of registered checkers. The <strong>Face Area Checker</strong> should appear there.</p>\n\
<p>Select it from the list to enable it.</p>\n\
<p><img src=\"./images/GUID-6149F943-BBB7-4D34-9BF2-8CE841F2F2B0-low.gif\" alt></p>\n\
<p>The label <strong>[ Face Area Checker ]</strong> will be displayed at the bottom of the screen and if a valid object (Editable Mesh or Editable Poly) is selected, the faces/polygons will be tested and shaded in the viewport.</p>\n\
<p><img src=\"./images/GUID-0CB1EB81-A26C-4625-8702-A628E4BF8B71-low.gif\" alt></p>\n\
<p>To configure the Min. and Max. Thresholds, click on the <strong>[ Click Here To Configure ]</strong> label.</p>\n\
<p><img src=\"./images/GUID-5D751E6C-E60B-4490-B68D-8AB67F2536A2-low.gif\" alt></p>\n\
<p>To make the checker available in future <strong>3ds Max</strong> sessions without the need to evaluate the script manually, copy the script to the \\Startup folder or any folder registered as a plugin path OR its sub-folders.</p>\n\
<p>We will take a look at how to add a custom xView Checker to the Viewport Menus in <a href=\"How-To-Develop-A-Face-Area-XView-3.html\">Part 4 of this tutorial</a>.</p>\n\
<p><strong>Next</strong></p>\n\
<p><a href=\"How-To-Develop-A-Face-Area-XView-1.html\">How To ... Develop A Face Area XView Checker - Part 2</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Develop-an-XView-Geometry.html\">How To ... Develop an XView Geometry Checker</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Tools-and-Interaction/Interacting-with-the-3ds-Max/Geometry-Checker/xView-Geometry-Checker-Access.html\">xView Geometry Checker Access</a></li>\n\
          <li><a class=\"link\" href=\"../../3ds-Max-Objects-and-Interfaces/Interfaces/Core-Interfaces/Core-Interfaces-Documentation/U-V-W-X-Y-Z/Interface-xViewChecker.html\">Interface: xViewChecker</a></li>\n\
          <li><a class=\"link\" href=\"../How-To-Introduction-to-the.html\">How To - Introduction to the Tutorials</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";