var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-36BA05E8-2F9A-4761-8D9B-40E3BC6503BF\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Try Expression</h1></div><p>MAXScript provides simple exception-handling with the <code>try</code> expression, a simplified form of the C++ exception-handling scheme.</p>\n\
<p>The <code>try</code> expression lets you bracket a piece of code and catch any runtime errors.</p>\n\
<p>This allows you to respond appropriately or take corrective action, rather than let MAXScript halt the execution of your script and print an error message.</p>\n\
<p>The <strong>syntax</strong> for the <code>try</code> expression is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">try &lt;protected_expr&gt; catch &lt;on_err_expr&gt; </code></pre><p>The <code>&lt;protected_expr&gt;</code> is executed and any errors that occur are trapped and the <code>&lt;on_err_expr&gt;</code> is executed.</p>\n\
<p>If the <code>&lt;protected_expr&gt;</code> is a block-expression, the block-expression stops executing at the point of the error.</p>\n\
<p>If there are no errors in the protected expression, the <code>&lt;on_err_expr&gt;</code> is not executed.</p>\n\
<p>This is a very simple error-trapping scheme, but limited heavily by the fact that you only get limited error codes or information about the error that occurred. See <a href=\"Try-Expression.html\"\">getCurrentException()</a> below.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   f = openFile &quot;foo.dat&quot;\n\
   try\n\
   while not eof f do read_some_data f\n\
   catch\n\
   (\n\
   messageBox &quot;bad data in foo.dat&quot;\n\
   results = undefined\n\
   )\n\
   close f\n\
</code></pre></blockquote>\n\
<p>In this example, any errors in reading or processing the data in the <code>read_some_data()</code> function are trapped, then a Message box is displayed and some clean-up is done.</p>\n\
<p>This is a good example of a use for the simple error-trapping.</p>\n\
<h2 id=\"throw\">Throw</h2><p>There is also an associated function, <code>throw()</code> , which can be used to generate your own runtime error and pass on an error you caught when doing some interim clean-up, or do a non-local jump.</p>\n\
<p>It has the form:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">throw &lt;error_message_string&gt; [ &lt;value&gt; ] [ debugBreak:&lt;boolean&gt; ]\n\
throw() </code></pre><p>If you have no intervening catches in your script, calling <code>throw()</code> with error message arguments will cause a runtime error to be signaled and MAXScript will report the error message using its standard error reporting.</p>\n\
<p>The optional <code>&lt;value&gt;</code> argument can be any value you want printed with the error message, perhaps the object involved in the error. This argument can not be specified if the <code>throw</code> is in the body of a <code>catch()</code> and will generate a compile error if present.</p>\n\
<p>The optional <code>debugBreak:&lt;boolean&gt;</code> argument specifies whether to break and display the MAXScript Debugger window. The default setting for this argument is set by the MAXScript Debugger configuration setting &quot;Default Throw debugBreak:on&quot;, which you can query with the <a href=\"./../../3ds-Max-Objects-and-Interfaces/Interfaces/Core-Interfaces/Core-Interfaces-Documentation/M/Interface-MXSDebugger.html\">MXSDebugger.defaultBreakOnThrow</a> property. Note that even if specified as true, if the &quot;Allow Break On Throw&quot; option is disabled for the debugger, the debugger will not display. You can query this setting with the <a href=\"./../../3ds-Max-Objects-and-Interfaces/Interfaces/Core-Interfaces/Core-Interfaces-Documentation/M/Interface-MXSDebugger.html\">MXSDebugger.allowBreakOnThrow</a> property.</p>\n\
<p>If the <code>try</code> / <code>catch</code> is within another <code>try</code> expression, calling <code>throw()</code> will cause the <code>catch</code> expression associated with the outer <code>try</code> expression to be executed.</p>\n\
<p>If you have no intervening catches in your script, calling <code>throw()</code> will cause MAXScript to continue running the script from just after the top-most set of parenthesis surrounding the <code>try</code> / <code>catch</code> , or will stop the currently running script if there are no parenthesis surrounding the <code>try</code> / <code>catch</code> .</p>\n\
<p>If you want to catch an error, perhaps to do some clean-up processing, and then pass the error on to outer handlers or MAXScript for error reporting, you can call <code>throw()</code> with <strong>no</strong> arguments.</p>\n\
<p>This throws the current error again and must <strong>only</strong> be done inside a <code>catch</code> expression.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   try\n\
   (\n\
   i=10\n\
   try ( i.x=1 ) -- will generate a run-time error\n\
   catch\n\
   (\n\
   print &quot;Bad Error&quot; -- error message printed\n\
   try (i.pos=[0,0,0]) -- will also generate a run-time error\n\
   catch (throw()) -- throw() will cause outer catch\n\
   -- to execute\n\
   )\n\
   )\n\
   catch (print &quot;Really Bad Error Occurred&quot;) -- error message printed\n\
</code></pre></blockquote>\n\
<h2 id=\"current-exception\">Current Exception</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getCurrentException() </code></pre><p>This method returns the text of the current exception.</p>\n\
<p>It is only useful inside a catch statement - if used outside a catch, it will return &apos;undefined&apos;.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   try\n\
   (\n\
   throw &quot;AAAAA&quot;\n\
   )\n\
   catch\n\
   (\n\
   format &quot;*** % ***\\n&quot; (getCurrentException())\n\
   )\n\
</code></pre><p>   <strong>RESULT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   *** -- Runtime error: AAAAA ***\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hasCurrentExceptionStackTrace()</code></pre><p><strong>Available since 3ds Max 2017</strong>: Returns true if a MAXScript stack trace is available. It is only useful inside a catch statement - if used outside a catch, it will return false. It should always return true within a catch.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hasCurrentExceptionCallStack()</code></pre><p><strong>Available since 3ds Max 2017</strong>: Returns true if a C++ stack trace is available. It is only useful inside a catch statement - if used outside a catch, it will return false. Within a catch, it will be true only if the exception that was caught was a system exception, such as an access violation or an integer divide by 0.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getCurrentExceptionStackTrace()</code></pre><p><strong>Available since 3ds Max 2017</strong>: Returns the MAXScript stack trace as a string. It is only useful inside a catch statement - if used outside a catch, it will return false. It should always return true within a catch.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getCurrentExceptionCallStack()</code></pre><p><strong>Available since 3ds Max 2017</strong>: Returns the C++ stack trace captured at the time an exception was thrown, as a string. It is only useful inside a catch statement - if used outside a catch, it will return &apos;undefined&apos;. Within a catch, if the C++ stack trace was not captured, an empty string is returned.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">CaptureCallStack [skipFirstNStackLevels:&lt;int&gt;]</code></pre><p><strong>Available since 3ds Max 2017</strong>: Captures the MAXScript call stack. The <code>skipFirstNStackLevels</code> parameter defaults to 3 if not specified.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;string&gt; getErrorSourceFileName()</code></pre><p><strong>Available since 3ds Max 2018</strong>: Returns the name of the source file causing the error, when called inside a catch statement. This can be useful if running code from multiple scripts, for example by using <code>fileIn()</code>.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This method is only useful inside a catch statement. If used outside a catch statement, it will return <code>undefined</code>.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;integer&gt; getErrorSourceFileLine()</code></pre><p><strong>Available since 3ds Max 2018</strong>: Returns the line number where the error occurred, when called inside a catch statement..</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This method is only useful inside a catch statement. If used outside a catch statement, it will return <code>undefined</code>.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;integer&gt; getErrorSourceFileOffset()</code></pre><p><strong>Available since 3ds Max 2018</strong>: Returns offset where the error occurred, when called inside a catch statement. This is the end of the expression being evaluated, including any comments.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This method is only useful inside a catch statement. If used outside a catch statement, it will return <code>undefined</code>.</div>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Controlling-Program-Flow.html\">Controlling Program Flow</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";