var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-35E5A5EF-BF14-4D11-8ABA-CFD9A0E7249E\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Defining Local Functions in Structures</h1></div><p>You can define local functions in structures, providing a convenient mechanism for packaging a set of related functions and avoiding possible name conflicts when many global functions are defined, possibly by several separate scripts.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE,</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   struct foo\n\
   (\n\
   a, b, c,\n\
   fn baz n = sqrt (n + 1),\n\
   fn bar x y = print (x ^ 2 + y ^ 2)\n\
   )\n\
</code></pre><p>   This defines a structure definition <code>foo</code> with three elements named <code>a</code> , <code>b</code> , and <code>c</code> , and two local functions: <code>bar</code> and <code>baz</code> . You can access these functions as properties of either the structure definition value itself or an instance of the foo structure:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   p = foo.baz q -- access as property of the structure definition value\n\
   foo.bar x y\n\
   f = foo 2 3 4\n\
   f.bar x y -- access as property of the structure instance\n\
</code></pre></blockquote>\n\
<p>Local functions declared in a structure definition can make references to the member data variables defined in the structure, such that when that member function in some instance of the structure is called, the references access the members of that instance. This allows you to initialize and maintain data structures for the structure functions within the structure itself. An example of including functions that use members of the structure is shown in the following script.</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   struct RandVals\n\
   (\n\
   RndVals=#(), ptr, eedVal,\n\
   fn generateRV num fromVal toVal =\n\
   (\n\
   if seedVal != undefined do seed seedVal\n\
   RndVals=for i=1 to num collect random fromVal toVal\n\
   ptr=1\n\
   RndVals\n\
   ),\n\
   fn deleteRv =\n\
   (\n\
   RndVals=#()\n\
   ptr=undefined\n\
   undefined\n\
   ),\n\
   fn getNextRV =\n\
   (\n\
   if Nvals == 0 do return undefined\n\
   val=RndVals[ptr]\n\
   ptr += 1\n\
   if ptr &gt;RndVals.count do ptr=1\n\
   val\n\
   ),\n\
   fn sortRV =(sort RndVals),\n\
   fn setSeed val = (SeedVal = val),\n\
   fn setPointer val =\n\
   (\n\
   if val &gt; 0 and val &lt;= RndVals.count then\n\
   (ptr=val;true)\n\
   else\n\
   false\n\
   )\n\
   )\n\
   MyRandomVals= RandVals()\n\
   MyRandomVals.setSeed 12345\n\
   MyRandomVals.generateRV 10 0 100\n\
   MyRandomVals.getNextRV()\n\
   MyRandomVals.sortRV()\n\
   MyRandomVals.setPointer 1\n\
   MyRandomVals.getNextRV()\n\
   MyRandomVals.deleteRv()\n\
</code></pre><p>   <strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   #Struct:RandVals( -- result of struct def lines 1 to 29\n\
     RndVals:&lt;data&gt;; Public,\n\
     sortRV:&lt;fn&gt;; Public,\n\
     ptr:&lt;data&gt;; Public,\n\
     seedVal:&lt;data&gt;; Public,\n\
     generateRV:&lt;fn&gt;; Public,\n\
     deleteRv:&lt;fn&gt;; Public,\n\
     getNextRV:&lt;fn&gt;; Public,\n\
     setSeed:&lt;fn&gt;; Public,\n\
     setPointer:&lt;fn&gt;; Public)\n\
   (RandVals RndVals:#() ptr:undefined seedVal:undefined) -- result line 30\n\
   12345 -- result line 31\n\
   #(23, 59, 79, 68, 17, 72, 84, 16, 89, 72) -- result line 32\n\
   23 -- result line 33\n\
   #(16, 17, 23, 59, 68, 72, 72, 79, 84, 89) -- result line 34\n\
   true -- result line 35\n\
   16 -- result line 36\n\
   undefined -- result line 37\n\
</code></pre></blockquote>\n\
<p>In this script, the structure has three data members and six function members. Note the references to the structure data members in the <code>generateRV()</code> function. In line 30, an instance of the <code>RndVals</code> structure is created and stored in variable <code>MyRandomVals</code> . In line 31, the <code>setSeed()</code> function in the structure instance is called, which initializes the random seed variable by storing the specified value in data member <code>seedVal</code> in the structure instance. In line 32, the <code>generateRV()</code> function in the structure instance is called, which initializes the random number using the seed value specified by <code>setSeed()</code> , sets up and stores an array of random numbers in data member <code>RndVals</code> , updates data member <code>ptr</code> , and then returns the random number array.</p>\n\
<p>Local functions defined in structures are not stored in every instance of the structure. The local function is stored once in the structure definition and references to them in the instances refer to their definitions in the structure definition.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Structure-Definition.html\">Structure Definition</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Structure-Definition-1.html\">Structure Definition</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Inherited-Methods.html\">Structure Inherited Methods</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Scope-While.html\">Structure Scope While Initializing</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Creation-and-Cloning.html\">Structure Creation and Cloning Event Handlers</a></li>\n\
          <li><a class=\"link\" href=\"Private-and-Public-Members-in.html\">Private and Public Members in Structure Definitions</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";