var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-C42F8CCE-CE31-49B4-9D5A-65C07D9F33FD\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Quat Values</h1></div><p>The <strong>Quat</strong> class provides a compact representation for orientation in three space and provides methods to perform Quaternion algebra.</p>\n\
<p>Quaternions are used to store object rotations in <strong>3ds Max</strong>.</p>\n\
<p>A quaternion is made up of four terms: a real scalar part that specifies the amount of rotation and an imaginary vector part that defines the axis of rotation.</p>\n\
<p>If the quaternion is normalized, the scalar term equals the cosine of half the angle of rotation, the vector term is the axis of rotation, and the magnitude of the vector term equals the sine of half the angle of rotation.</p>\n\
<p>Interpolation between two key frame orientations is much easier using quaternions and produces smooth and natural motion. Unlike Euler angles, no numerical integration is necessary; quaternions provide an analytic result (no approximations).</p>\n\
<p>Rotations in MAXScript follow the right-hand-rule, namely positive angles rotate counter-clockwise about positive axes, consistent with the convention in the <strong>3ds Max</strong> UI.</p>\n\
<p><strong>Constructors</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">quat &lt;x_float&gt; &lt;y_float&gt; &lt;z_float&gt; &lt;w_float&gt;</code></pre><p>The x, y, z values make up the vector portion. w is the angle of rotation about the vector.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">quat &lt;degrees_float&gt; &lt;axis_point3&gt;\n\
&lt;angleaxis&gt; as quat\n\
&lt;eulerangle&gt; as quat\n\
&lt;matrix3&gt; as quat --extracts the rotation component as a quat</code></pre><p><strong>Properties</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;quat&gt;.w: Float\n\
&lt;quat&gt;.x: Float\n\
&lt;quat&gt;.y: Float\n\
&lt;quat&gt;.z: Float</code></pre><p>Access quaternion complex components as Floats.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;quat&gt;.angle: Float\n\
&lt;quat&gt;.axis: Point3</code></pre><p>Access derived properties (an angle in degrees and a rotation axis).</p>\n\
<p><strong>Operators</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;quat&gt; + &lt;quat&gt;\n\
&lt;quat&gt; - &lt;quat&gt;\n\
&lt;quat&gt; * &lt;quat_or_number&gt;\n\
&lt;quat&gt; / &lt;number&gt;\n\
- &lt;quat&gt;</code></pre><p>Quaternion algebra.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The <code>+</code> operator is the same as the <code>*</code> operator (product) and the <code>-</code> operator is the same as the <code>/</code> operator (ratio).</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;quat&gt; * &lt;matrix3&gt;</code></pre><p>Quaternion algebra using rotation portion of the matrix3.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> Multiplication with quaternions is not communicative. When multiplying an angle, for example, the quat must be the first term followed by the integer or float.  </div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;quat&gt; == &lt;quat&gt;\n\
&lt;quat&gt; != &lt;quat&gt;\n\
&lt;quat&gt; as &lt;class&gt;</code></pre><p>Quats can convert to Matrix3&apos;s, Angleaxis&apos;s, and Eulerangle&apos;s.</p>\n\
<p><strong>Methods</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">copy &lt;quat&gt;</code></pre><p>Creates a new copy of the quat value.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   newQuat = copy oldQuat</code></pre></blockquote>\n\
<p>The new value contains a copy of the input quat value, and is independent of the input quat value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">isIdentity &lt;quat&gt;</code></pre><p>Returns <code>true</code> if the quaternion is equal to the identity quaternion (x=y=z=0.0; w=1.0).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">normalize &lt;quat&gt;</code></pre><p>Returns a normalized quat, dividing each term of the input quat by a scale factor such that the resulting sum of the squares of all parts equals unity.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">inverse &lt;quat&gt;</code></pre><p>Returns the inverse of the quaternion (1/q).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">conjugate &lt;quat&gt;</code></pre><p>Returns the conjugate of the quaternion.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">logN &lt;quat&gt;</code></pre><p>Returns the natural logarithm of UNIT quaternion.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">exp &lt;quat&gt;</code></pre><p>Returns the exponentiated quaternion (where q.w=0).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">slerp &lt;start_quat&gt; &lt;end_quat&gt; &lt;number&gt;</code></pre><p>Returns a spherical linear interpolation of UNIT quaternions.</p>\n\
<p>As <code>&lt;number&gt;</code> goes from 0 to 1, the returned value goes from <code>&lt;start_quat&gt;</code> to <code>&lt;end_quat&gt;</code>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">LnDif &lt;p_quat&gt; &lt;q_quat&gt;</code></pre><p>Returns the &quot;log difference&quot; of two quaternions as:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">logN ((inverse p_quat)*q_quat)</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">qCompA&lt;prev_quat&gt; &lt;now_quat&gt; &lt;next_quat&gt;</code></pre><p>Computes <code>a</code> , the term used in Boehm-type interpolation:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">a = now_quat * exp(-(1/4)*(logN((inverse now_quat)*next_quat)+logN((inverse now_quat)*prev_quat)))</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">squad&lt;p_quat&gt; &lt;a_quat&gt; &lt;b_quat&gt; &lt;q_quat&gt; &lt;number&gt;</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">slerp (slerp p_quatq_quatnumber) (slerp a_quatb_quatnumber) (2*(1-number)*number)</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">qorthog &lt;quat&gt; &lt;axis_point3&gt; </code></pre><p>Returns <code>quat</code> rotated by 180 degrees (quaternion space metric) about the specified axis <code>axis_point3</code> .</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">transform &lt;quat&gt; &lt;matrix3&gt;</code></pre><p>Returns the transformation of the specified quaternion by the specified matrix.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">squadrev &lt;angle_number&gt; &lt;axis_point3&gt; &lt;start_quat&gt; &lt;start_tangent_quat&gt; &lt;end_tangent_quat&gt; &lt;end_quat&gt; &lt;number&gt;</code></pre><p>Returns quaternion interpolation for angles &gt; 2PI, where <code>angle_number</code> is the angle of rotation, and <code>axis_point3</code> is the axis of rotation. As <code>number</code> goes from 0 to 1, the returned value goes from <code>start_quat</code> to <code>end_quat</code> .</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">random &lt;quat&gt; &lt;quat&gt;</code></pre><p>Returns a random rotation between the two quats, using Slerp.</p>\n\
<p><strong>Related Methods</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">quatToEuler&lt;quat&gt; order:&lt;eulertype_integer&gt;\n\
eulerToQuat&lt;eulerAngle&gt; order:&lt;eulertype_integer&gt;</code></pre><p>Converts between quat and euler angle values. The optional order parameter specifies the order of application of the angles. If not specified, XYZ ordering is used. Its value can be any of the following:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">1 - XYZ\n\
2 - XZY\n\
3 - YZX\n\
4 - YXZ\n\
5 - ZXY\n\
6 - ZYX\n\
7 - XYX\n\
8 - YZY\n\
9 - ZXZ\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">quatToEuler2 &lt;quat&gt;</code></pre><p>Returns the same Euler value for the given Quaternion as shown in the Transform Type-In dialog. Available in <strong>3ds Max 2008 and higher</strong>. Previously, available in the <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a> .</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">quatArrayToEulerArray &lt;quat array&gt;</code></pre><p>Returns an array of smooth Euler values from the Quat array. Available in <strong>3ds Max 2008 and higher</strong>. Previously, available in the <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getEulerQuatAngleRatio &lt;quat1&gt; &lt;quat2&gt; &lt;eulerAngles1&gt; &lt;eulerAngles2&gt; [angle:&lt;eulertype_integer&gt;]</code></pre><p>When converting a series of quat values to eulerAngles values, it is possible for sign flips to occur in the eulerAngles values. This is due to the fact that one single quat value can be expressed through many different eulerAngles values. This flip can be detected based on the eulerAngles/quat ratio. The eulerAngles/quat ratio is the relation of the angle difference in eulerAngles space to the angle difference in quat space. If this ratio is bigger than PI, the rotation between the two quat to eulerAngles conversions. This method returns the eulerAngles/quat angle ratio between the two quat to eulerAngles conversions as a float value. The actual detection of the flip is dependent on the amount of rotation in between conversions. The smaller the amount of rotation, the more accurate the detection is. The parameters to this method are:</p>\n\
<p><code>quat1</code> and <code>quat2</code> are the previous and current rotation values.</p>\n\
<p><code>eulerAngles1</code> and <code>eulerAngles2</code> are the previous and current converted rotation angles.</p>\n\
<p>The optional <code>eulertype_integer</code> parameter specifies the order of application of the angles. If not specified, XYZ ordering is used. Its value can be any of the following:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">1 - XYZ\n\
2 - XZY\n\
3 - YZX\n\
4 - YXZ\n\
5 - ZXY\n\
6 - ZYX\n\
7 - XYX\n\
8 - YZY\n\
9 - ZXZ\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Basic-Data-Values.html\">Basic Data Values</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"AngleAxis-Values.html\">AngleAxis Values</a></li>\n\
          <li><a class=\"link\" href=\"EulerAngles-Values.html\">EulerAngles Values</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";