var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-B015A2D1-AC4D-4773-99F1-78115ACC944D\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Working with MAXKey Values</h1></div><p>Changing the <code>.time</code> property of a key may cause it to go out of time order relative to the other keys in the controller. You <strong>must</strong> call the <code>sortKeys()</code> function on the controller or associated <strong>MAXKeyArray</strong> once all key time manipulations of this kind is complete so that animation will perform correctly.</p>\n\
<p>The class of a key&apos;s value property is determined by the controller it is in, for example, a <code>linear_float</code> key has a float <code>.value</code> , a <code>tcb_rotation</code> key has a quaternion <code>.value</code> , etc. The properties are all settable and support the math-assignment operators and nested-property access.</p>\n\
<blockquote>\n\
<p>   <strong>HERE ARE SOME EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $foo.pos.keys[2].time += 20f -- change time of single key\n\
   $foo.pos.keys.time += 20f -- change time of all keys\n\
   $box1.uvw_map.center.keys[2].value.x += 20\n\
   for k in $baz.bend.angle.keys do k.value *= 1.1\n\
</code></pre></blockquote>\n\
<p>In some cases, the actual value stored in a key is not the value shown in the command panels, shown in Track View, the value returned by the <code>.value</code> property of the key, or the value returned when accessing the property to which the key&#x2019;s controller is assigned. Instead, a scaling factor is applied to the key&#x2019;s value before the value is made &quot;visible&quot;. For example, percentages are normally displayed with a range of 0-100, rotation angles are in degrees, colors are [0-255,0-255,0-255]. The unscaled key values for percentages are normally in a range of 0-1, rotation angles in radians, and colors are [0-1,0-1,0-1].</p>\n\
<p>In the description of properties for each class, the scaling factor, if any, is identified for each property. Take the following properties for the PArray class</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   &lt;PArray&gt;.X_Spin_Vector Float default: 1.0 -- animatable\n\
   &lt;PArray&gt;.Spin_Time_Variation Float default: 0.0 -- animatable, percentage\n\
   &lt;PArray&gt;.Spin_Phase Float default: 0.0 -- animatable, angle\n\
   &lt;PArray&gt;.viewPercent Float default: 10.0 -- percentage\n\
</code></pre></blockquote>\n\
<p>The <code>.X_Spin_Vector</code> property does not apply any scaling to its controller keys. The <code>.Spin_Time_Variation</code> property applies percentage scaling to its controller keys (displayed as percentage, stored as fraction). The <code>.Spin_Phase</code> property applies angle scaling to its controller keys (displayed as degrees, stored as radians). The <code>.viewPercent</code> property is stored internally with a percentage scaling factor, but since this property is not animatable, this scaling it invisible to MAXScript.</p>\n\
<p>Normally this scaling is not of concern when programming in MAXScript, but there are two cases where it must be taken into account. The first is when using script controllers, and is described in <a href=\"./../../../3ds-Max-Objects-and-Interfaces/Animation-Controllers/Controller-Types/Script-Controllers/Script-Controllers-1.html\">Script Controllers</a>.</p>\n\
<p>The second case is when one controller is applied as the controller for two or more properties. The scaling is an attribute of the animatable <strong>property</strong>, not the controller, so in (the admittedly rare) situations in which a controller is shared by several animatables that have <strong>different</strong> scaling, unexpected results will occur if you do not take into account the scaling factor. An example of this is shown in the following script, where the same controller is shared between three differently scaled properties.</p>\n\
<blockquote>\n\
<p>   <strong>FOREXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   obj=PArray()\n\
   cont=bezier_float()\n\
   obj.X_Spin_Vector.controller=cont\n\
   obj.Spin_Time_Variation.controller=cont\n\
   obj.Spin_Phase.controller=cont\n\
   obj.X_Spin_Vector=1\n\
   obj.X_Spin_Vector -- no scaling\n\
   obj.Spin_Time_Variation -- percentage\n\
   obj.Spin_Phase -- angle\n\
</code></pre><p>   <strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $PArray:PArray03 @ [0,0,0] -- result line 1\n\
   Controller:Bezier_Float -- result line 2\n\
   Controller:Bezier_Float -- result line 3\n\
   Controller:Bezier_Float -- result line 4\n\
   Controller:Bezier_Float -- result line 5\n\
   1 -- result line 6\n\
   1.0 -- result line 7, unscaled\n\
   100.0 -- result line 8, percentage scaled, 1 = 100%\n\
   57.2958 -- result line 9, angle scaled, \n\
   1 radian = 57.2958 degrees \n\
</code></pre></blockquote>\n\
<p>If you do assign the same controller to properties with different scaling factors, you should take care to access the <code>.value</code> property through the correct animatable property - this provides a scaling context for the access operation.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../MAXKey-Values.html\">MAXKey Values</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";