var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-DF09BF99-2106-408D-9965-60436B853D76\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Memory Allocation and Garbage Collection</h1></div><p>MAXScript allocates its own working memory in addition to any memory <strong>3ds Max</strong> allocates as it works.</p>\n\
<p>The default working memory size setting for MAXScript is 100 MB (as of <strong>3ds Max 2021.3 Update</strong>), and the memory is virtual and pageable like all <strong>3ds Max</strong> memory.</p>\n\
<p>The default allocation accommodates most tasks, but you can increase it by incrementing the MAXScript system global variable <code>heapSize</code> :</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   heapSize += 50 * 1024 * 1024 -- increase the heap size by 50 MB\n\
</code></pre></blockquote>\n\
<p>You can add this statement to your startup script if you always need extra memory allocated for MAXScript tasks or you can set the default size in the MAXScript tab in the <strong>3ds Max</strong> Customize&gt;Preferences dialog.</p>\n\
<p>Increasing the MAXScript heap reduces memory address space for other <strong>3ds Max</strong> tasks.</p>\n\
<p>Adjust the default memory setting only after you receive &quot;Out of memory&quot; error messages from MAXScript, or when garbage collection pauses occur frequently.</p>\n\
<p>You can increase the MAXScript heap as many times as you need during a <strong>3ds Max</strong> session, but you cannot reduce it.</p>\n\
<p>Restart <strong>3ds Max</strong> to reset the MAXScript memory heap to the default setting.</p>\n\
<p>MAXScript uses a memory management scheme called <strong>automatic garbage collection</strong>.</p>\n\
<p>This means you don&#x2019;t have to explicitly reclaim any memory that a value takes up when you no longer need it.</p>\n\
<p>When you assign a new value to a variable or when a local variable&#x2019;s block exits, the reference that the variable contained is dropped.</p>\n\
<p>When your program has dropped all references to a value&apos;s memory, that memory becomes eligible for reclamation.</p>\n\
<p>Some values, such as arrays, have internal references to other values.</p>\n\
<p>In an array, each element contains a reference to another value.</p>\n\
<p>When you assign a new value to an array element, the element&apos;s old reference is dropped.</p>\n\
<p>If you remove all references to the array itself, all the references in the elements of that array are implicitly dropped.</p>\n\
<p>The same applies to component values, such as in user structures and most <strong>3ds Max</strong> objects.</p>\n\
<p>MAXScript reclaims memory in a garbage collection pass through memory when MAXScript runs out of memory.</p>\n\
<p>This simple approach can result in a short pause because the entire MAXScript memory is scanned for garbage.</p>\n\
<p>The advantage of garbage collection is you can create as many values as you need, and MAXScript cleans up the ones you no longer use.</p>\n\
<p>In <strong>3ds Max 9</strong>, the <strong><code>HeapSize</code> and the <code>HeapFree</code></strong> system variables were changed from Integer to <strong>Integer64</strong> to accommodate the increased memory address space of 64 bit systems.</p>\n\
<p>In <strong>3ds Max 9 and higher</strong>, the initial heap size is limited to 512MB.</p>\n\
<p>If the initial heap size is still too high and the specified memory block cannot be allocated, the initial heap size is recursively decreased by 10% until the memory block can be allocated.</p>\n\
<p>If a 1MB memory block cannot be allocated, an assert is triggered and the <strong>3ds Max</strong> session is terminated (if the system cannot allocate 1MB at startup, something is very wrong).</p>\n\
<p>Otherwise, a message is displayed in the status panel stating that the requested heap size was automatically decreased, and gives the actual heap size allocated.</p>\n\
<p>A test was added to ensure that the initial heap size is at least the absolute minimum required by MAXScript - 12 bytes.</p>\n\
<p>Code was added so that as new values are allocated, the heap size is automatically increased if necessary (requested memory size not available in the heap after performing a full garbage collection) .</p>\n\
<p>The first attempted increase is the maximum of 50* the requested size or 512KB.</p>\n\
<p>If this allocation fails, a series of allocation attempts is made with decreasing allocation sizes, with the last one being the maximum of the requested size or 256KB.</p>\n\
<p>If this final allocation attempt fails, a runtime error is thrown.</p>\n\
<p>Otherwise, a message is displayed in the status panel stating that the requested heap size was automatically increased, and gives the new heap size.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">heapCheck()      </code></pre><p>The <strong><code>heapCheck()</code></strong> method is mostly for internal debugging.</p>\n\
<p>It can be called by scripts to perform a minimal consistency check of the Heap.</p>\n\
<p>Available in <strong>3ds Max 9 and higher</strong>.</p>\n\
<p>Possible return values are:</p>\n\
<p><strong>OK</strong> - Heap appears to be consistent (_HEAPOK)</p>\n\
<p><strong>-1</strong> - Heap has not been initialized (_HEAPEMPTY)</p>\n\
<p><strong>-3</strong> -Initial header information is bad or cannot be found.(_HEAPBADBEGIN)</p>\n\
<p><strong>-4</strong> -Bad node has been found or heap is damaged.(_HEAPBADNODE)</p>\n\
<p><strong>-6</strong> - Pointer into Heap is not valid (_HEAPBADPTR)</p>\n\
<p>See also the <a href=\"./../../MAXScript-Introduction/The-MAXScript-Debugger/CRT-Debug-Functions.html\">CRT Debug Functions</a> topic for related methods.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><strong><a href=\"Memory-Allocation-and-Garbage/Manual-Garbage-Collection.html\">Manual Garbage Collection</a></strong></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Variables-Assignment-and-Scope.html\">Variables - Assignment and Scope</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Memory-Allocation-and-Garbage/Manual-Garbage-Collection.html\">Manual Garbage Collection</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Introduction/The-MAXScript-Debugger/CRT-Debug-Functions.html\">CRT Debug Functions</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";