var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-382E7583-DD2A-4DF5-B568-71502DF95ED9\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Scope of Variables</h1></div><p>Variables have an attribute called <em>scope</em> that determines where a variable can be accessed in MAXScript code.</p>\n\
<p>MAXScript has two kinds of variable scope, global and local.</p>\n\
<p><em>Global variables</em> are visible in all running MAXScript code and hold their values until you exit <strong>3ds Max</strong>.</p>\n\
<p><em>Local variables</em> are directly visible to code only in the lexical scope currently in effect, and hold their values only as long as the current scope is active. After the scope is no longer active, you can no longer access the contents of a variable local to that scope. This is similar to how most modern programming languages implement variables. The conditions under which MAXScript creates a new scope are described <a href=\"Scope-of-Variables.html\"\">further on this page</a>.</p>\n\
<h2 id=\"explicit-variable-declaration\">Explicit Variable Declaration</h2><p>MAXScript provides language constructs used to <strong>explicitly</strong> declare and initialize both global and local variables.</p>\n\
<p>The syntax for the variable declaration ( <code>&lt;variable_decls&gt;</code> ) is as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">( local | global ) &lt;decl&gt; { , &lt;decl&gt; }</code></pre><p>where <code>&lt;decl&gt;</code> is defined as:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;var_name&gt; [ = &lt;expr&gt; ] -- name and optional initial value</code></pre><p><strong>EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">global baz -- initialized to &apos;undefined&apos;\n\
global foo=10, initialized=false -- initialized globals\n\
local x = 1, -- continued on several lines\n\
y = 2,\n\
z = sqrt(123.7)\n\
</code></pre><p>As mentioned above, local variables hold their values as long as the scope is active. In nested scopes, this is usually a very short period. It starts when a function or loop or block expression runs and ends when it exits.</p>\n\
<p>Each time the scope is entered at runtime, a new set of locals is created and then retired when the scope exits.</p>\n\
<p>It is a good practice to always declare variables explicitly. See \n\
<a href=\"Scope-of-Variables.html\"\">further on this page</a> for reasons.</p>\n\
<h2 id=\"explicit-global-variable-access\">Explicit Global Variable Access</h2><p>Since <strong>3ds Max 8</strong>, MAXScript provides a way to explicitly access a variable name as global. If the variable exists in the global pool of variables, it will be used. If the variable name does not exist in the global scope, it will be created as explicitly global.</p>\n\
<p>The syntax is double-colon &apos;::&apos; and is <a href=\"Specifying-Global-Variables-As.html\">described in this topic</a>.</p>\n\
<p><a name=\"local-variables-as--private-global--variables\"></a></p>\n\
<h2 id=\"local-variables-as-39-private-global-39-variables\">Local Variables as &apos;Private Global&apos; variables</h2><p>In certain situations, the top-level scope can remain active for extended periods, so the locals declared at the top-level hold their values for that extended period.</p>\n\
<p>In particular, top-level locals declared in scripted rollouts, utilities, plug-ins, script controllers, and Macro Scripts hold their values for as long as the rollout, utility, plug-in, or Macro Script exists.</p>\n\
<p>Typically, they remain in existence until you redefine them, for example, when you define a Macro Script and run it for the first time, the top-level scope is created and remains active over any number of subsequent executions unless you redefine the Macro Script. At this point, the existing top-level scope is retired and a fresh one (along with its top-level locals) is created.</p>\n\
<p>This lets you use top-level locals in these constructs as a kind of <strong>private global</strong>; the values they hold remain in place over many executions, but only the code in that construct can see the variable, so it does not conflict with actual global variables.</p>\n\
<p>Note that in scripted plug-ins, a separate copy of the top-level local scope is created for <em>each</em> instance of the plugin and this scope remains active while that instance remains in existence up until the end of the current <strong>3ds Max</strong> session.</p>\n\
<p>*<em>HISTORY Note: *</em></p>\n\
<p>In <strong>3ds Max</strong> R2.x, the optional initialization value was applied to global variables only when the declaration creates a new variable. For example, if you ran the following script twice, the value of x in the second line will be 10 for the first execution, and 20 for the second execution. In <strong>3ds Max 3</strong> and later, the optional initialization value is always applied to global variables.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">global x = 10\n\
print x\n\
x=20</code></pre><p>If you want to make the global variable initialization conditional in <strong>3ds Max 3 and higher</strong>, use a construct similar to:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">global foo; if foo = = undefined do foo = 23</code></pre><p><a name=\"implicit-variable-declaration\"></a></p>\n\
<h2 id=\"implicit-variable-declaration\">Implicit Variable Declaration</h2><p>If you do not explicitly declare a variable and the variable name does not exist at a higher scope, MAXScript will create the variable the first time you use it and initialize it to hold the special value <code>undefined</code> . You are not required to explicitly declare a variable or initialize its value before you can use it. Variable names not explicitly declared by one of the previous statements are called implicitly declared variables. The scope of implicitly declared variables is the MAXScript scope context that is currently in effect when the variable is first used. The initial MAXScript scope context is global, and new scope contexts are opened for the following:</p>\n\
<ul>\n\
<li><p>Top-level open parentheses in a script file or in the Listener</p>\n\
</li>\n\
<li><p>Start of a function body</p>\n\
</li>\n\
<li><p>Start of a <code>for</code> loop body</p>\n\
</li>\n\
<li><p>Start of a utility, rollout, right-click menu, Macro Script, or tool definition</p>\n\
</li>\n\
<li><p>Start of a rollout, utility, right-click menu, Macro Script, or tool event handler</p>\n\
</li>\n\
<li><p>Start of a <code>when</code> body</p>\n\
</li>\n\
</ul>\n\
<p>Within these new scopes, newly referenced variables will be implicitly declared as local variables.</p>\n\
<p>Scope contexts are nested with the scope of a variable explicitly or implicitly declared at one scope context level extending to all scope contexts below that level.</p>\n\
<p>Take for example the following script:</p>\n\
<table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\"><colgroup><col width=\"15.356711003627572%\"> <col width=\"84.64328899637243%\"> </colgroup><tbody><tr class=\"ruled-odd-row\"><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86914\"></a><p class=\"table-body\"><em class=\"strong\">LINE #</em></p></td><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86915\"></a><p class=\"table-body\"><em class=\"strong\">EXAMPLE</em></p></td></tr><tr class=\"ruled-even-row\"><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">1\n\
2\n\
3\n\
4\n\
5\n\
6\n\
7\n\
8\n\
9\n\
10\n\
11\n\
</code></pre></td><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">a=10 -- scope context: global\n\
( b=20 -- new scope context: level 1\n\
for i = 1 to 5 do -- new scope context: level 2\n\
( j=random i a\n\
k=random i b\n\
print (j*a+k*b)\n\
) -- end of scope context: level 2\n\
a=a+b\n\
) -- end of scope context: level 1\n\
print a -- back to global scope context\n\
print k\n\
</code></pre></td></tr><tr class=\"ruled-odd-row\"><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E8692F\"></a><p class=\"table-body\"><em class=\"strong\">RESULT #1</em></p></td><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86930\"></a><p class=\"table-body\"><em class=\"strong\">COMMENTS</em></p></td></tr><tr class=\"ruled-even-row\"><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">10\n\
170\n\
120\n\
290\n\
320\n\
360\n\
30\n\
30\n\
30\n\
undefined\n\
undefined\n\
</code></pre></td><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">result of a=10\n\
print from i=1\n\
print from i=2\n\
print from i=3\n\
print from i=4\n\
print from i=5\n\
return value of end of scope level 1\n\
result of print a\n\
return value of print a\n\
result of print k\n\
return value of k</code></pre></td></tr><tr class=\"ruled-odd-row\"><td class=\"table-body\"><a name=\"P_E502A0A6852C4D07B1530DC58F9A1674\"></a><p class=\"table-body\"><em class=\"strong\">RESULT #2</em></p></td><td class=\"table-body\"><a name=\"P_95DFF8106E0D4C4881148830B5BFDC9E\"></a><p class=\"table-body\"><em class=\"strong\">COMMENTS</em></p></td></tr><tr class=\"ruled-even-row\"><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">10\n\
170\n\
420\n\
410\n\
180\n\
160\n\
30\n\
30\n\
30\n\
5\n\
5\n\
</code></pre></td><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">result of a=10\n\
result of print from loop i=1\n\
result of print from loop i=2\n\
result of print from loop i=3\n\
result of print from loop i=4\n\
result of print from loop i=5\n\
return value at end of scope level 1\n\
result of print a\n\
return value of print a\n\
result of print k\n\
return value of k</code></pre></td></tr></tbody></table>\n\
\n\
\n\
<p>In the above script, variable <code>a</code> is first used in the global scope context and its scope includes scope contexts global, level 1, and level 2.</p>\n\
<p>Variable <code>b</code> is first used in scope context level 1, so it is implicitly declared as a local variable and its scope includes scope contexts level 1 and 2.</p>\n\
<p>Variables <code>i</code> and <code>j</code> are first used in scope context level 2 and their scope is only scope context level 2.</p>\n\
<p>The scope of variable <code>k</code> varies depending on whether this script has been run before.</p>\n\
<p>The first time the script is run, variable <code>k</code> is first defined at line 5 and its scope is scope context level 2.</p>\n\
<p>In line 11, variable <code>k</code> is used again. Because the variable <code>k</code> defined in line 5 is no longer in scope, a new variable <code>k</code> is defined whose scope is global.</p>\n\
<p>The second time you run the script, at line 5 MAXScript detects that variable <code>k</code> already exists and uses it.</p>\n\
<p>So, the <strong>first time</strong> you run the script, line 11 will print <code>undefined</code> , and the <strong>second time</strong> you run the script, line 11 will print the last value calculated at line 5.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The scope of the variable used as a <code>for</code> loop counter (variable <code>i</code> in the above script) is a special case. The <code>for</code> loop counter variable&apos;s scope is always the scope context created for the <code>for</code> loop. This is true even if the <code>for</code> loop counter variable&apos;s name has already been implicitly or explicitly declared at a higher scope context level.</div>\n\
<p><a name=\"lexical-scoping\"></a></p>\n\
<h2 id=\"lexical-scoping\">Lexical Scoping</h2><p>When you explicitly declare a local variable, you can reuse the same name as a variable at a higher scope context level.</p>\n\
<p>If you do this, the newly declared local variable hides the outer variables with the same name.</p>\n\
<p>Any reference to that variable name later in the local variable&apos;s scope refers to the new local variable.</p>\n\
<p>At the end of the local variable&apos;s scope, the next outer variable becomes visible again.</p>\n\
<p>This visibility scheme is called <em>lexical scoping</em>. An example of lexical scoping is shown in the following script.</p>\n\
<table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\"><colgroup><col width=\"48.45360824742268%\"> <col width=\"51.54639175257733%\"> </colgroup><tbody><tr class=\"ruled-odd-row\"><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86977\"></a><p class=\"table-body\">#</p></td><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86978\"></a><p class=\"table-body\"><em class=\"strong\">EXAMPLE</em></p></td></tr><tr class=\"ruled-even-row\"><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">1\n\
2\n\
3\n\
4\n\
5\n\
6\n\
7\n\
8\n\
9\n\
10\n\
11\n\
12\n\
13\n\
14\n\
15\n\
16\n\
17\n\
18\n\
19\n\
20\n\
21\n\
22\n\
23\n\
24\n\
25\n\
26\n\
</code></pre></td><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">global foo = 23, x = 20 -- explicit declaration of foo and x,\n\
-- scope is global\n\
y = 10 -- implicit declaration of y,\n\
-- scope is global\n\
format &quot;context level global: foo= %\\n&quot; foo\n\
if x &gt; y then\n\
( -- top-level open parentheses\n\
-- new scope is created\n\
local baz = foo + 1 -- uses the global foo\n\
local foo = y - 1 -- declares 1st local foo, hides global foo\n\
format &quot;context level 1: foo= %\\n&quot; foo\n\
b=box() -- b implicitly declared as local\n\
b.pos.x = foo -- uses 1st local foo\n\
if (foo &gt; 0) then\n\
(\n\
local a\n\
local foo = y - x -- a nested 2nd local foo, hides 1st local\n\
format &quot; context level 2: foo= %\\n&quot; foo\n\
a = sin foo -- uses 2nd local foo\n\
format &quot;a= %\\n&quot; a\n\
) -- leave scope\n\
b.pos.y= foo - 1.5 -- back to 1st local foo\n\
format &quot;context level 1: foo= %\\n&quot; foo\n\
) -- leave scope\n\
-- b, baz and foo no longer in scope\n\
format &quot;context level global: foo= %\\n&quot; foo -- back to global foo\n\
</code></pre></td></tr><tr class=\"ruled-odd-row\"><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E869B0\"></a><p class=\"table-body\">&#xA0;</p></td><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E869B1\"></a><p class=\"table-body\"><em class=\"strong\">OUTPUT</em></p></td></tr><tr class=\"ruled-even-row\"><td class=\"table-body\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E869B2\"></a><p class=\"table-body\">&#xA0;</p></td><td class=\"table-body\"><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">20 -- result of line 1\n\
10 -- result of line 3\n\
context level global - foo= 23 -- output from line 5\n\
OK -- result of line 5\n\
context level 1: foo= 9 -- output from line 11\n\
context level 2: foo= -10 -- output from line 18\n\
a= -0.173648 -- output from line 20\n\
context level 1: foo= -10 -- output from line 23\n\
OK -- result of if expression lines 6 to 24\n\
context level global: foo= 23 -- output from line 26\n\
OK -- result of line 26\n\
</code></pre></td></tr></tbody></table>\n\
\n\
\n\
<p>This might seem a strange way of over-using a single variable name, but these scoping rules can be useful in large programs.</p>\n\
<p>For example, you might add a new user-interface item and its handlers to a utility script. By explicitly declaring the variables used in the handlers as local, you ensure these variables are independent of any pre-existing variables with the same names in the remainder of the script.</p>\n\
<p>When writing scripts, it is <strong>good programming practice</strong> to <strong>explicitly declare</strong> your local and global variables.</p>\n\
<p>Implicit declaration is provided as a <strong>short-hand</strong>, typically used when working in the Listener interactively or developing short scripts.</p>\n\
<p>When developing extended scripts, <strong>explicitly declaring</strong> variables can <strong>reduce errors</strong> and improve readability of the code.</p>\n\
<p>It is also recommended that you declare as local all variables unless you really want them to be global variables. There are several reasons for this practice:</p>\n\
<ul>\n\
<li><p>All the variable names used in the block or function will be together, which makes it easier to find the variable names and helps prevent using incorrect names in the script.</p>\n\
</li>\n\
<li><p>If more than one script is executed at the same time (for example, you are running a scripted utility and have a scripted controller in the scene), and both scripts use the same global variable name, the two scripts can interfere with one another. This can cause random failures of one or both scripts.</p>\n\
</li>\n\
<li><p>You are sure of using the correct variable in the event that a variable with the same name has already been declared at a higher scope, and you cannot inadvertently change the higher-level variable&apos;s value.</p>\n\
</li>\n\
<li><p>Values of variables explicitly defined as local are displayed in error call stack trace-backs.</p>\n\
</li>\n\
<li><p>In <strong>3ds Max</strong> R2.5, if a <strong>3ds Max</strong> class, a MAXScript method, or a MAXScript read-only global variable has the same name as an implicitly declared local variable, a MAXScript runtime error will be generated if an assignment to that variable name occurs. Because the variable name already existed with a global scope, an implicitly declared local variable is not created. As one or more new <strong>3ds Max</strong> classes will be created by any third party plug-ins the user has loaded, you cannot tell ahead of time if a variable name is a global variable name. In <strong>3ds Max</strong> R3, if the MAXScript compiler detects that the first reference to such a variable is an assignment, it will implicitly declare a local variable, rather than leaving it as a reference to a read-only system global. For example, executing the following expression in <strong>3ds Max</strong> R2.5 results in a value of <code>Box</code> because <code>Box</code> is a <strong>3ds Max</strong> object class. In 3DS R3, MAXScript detects that the first use of variable <code>box</code> is an assignment and creates an implicitly declared local variable <code>box</code> . The result of this expression in <strong>3ds Max</strong> R3 is the value <code>undefined</code> .</p>\n\
</li>\n\
</ul>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">(if false do box=10; box)</code></pre><p>In the following script, an error was introduced by using undefined variable <code>k</code> in line 7. In the output, the error call stack trace-back shows the value for variable <code>b</code> in function <code>afunc</code> and in the block-expression calling <code>afunc</code> .</p>\n\
<p><strong>SCRIPT</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn afunc =\n\
(\n\
local b\n\
b = box()\n\
for i in 1 to 10 do\n\
for j in 1 to 10 do\n\
instance b pos:[i*20,j*30,30*sin(k*36)]\n\
)\n\
--\n\
(\n\
global c=10\n\
local b\n\
b=&quot;hello&quot;\n\
afunc()\n\
)\n\
</code></pre><p><strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">afunc()\n\
-- Error occurred in j loop\n\
-- Frame:\n\
-- k: undefined\n\
-- j: 1\n\
-- called in i loop\n\
-- Frame:\n\
-- i: 1\n\
-- called in afunc()\n\
-- Frame:\n\
-- b: $Box:Box102 @ [0.000000,0.000000,0.000000]\n\
-- called in &lt;block&gt;()\n\
-- Frame:\n\
-- b: &quot;hello&quot;\n\
-- No &quot;&quot;*&quot;&quot; function for undefined\n\
OK\n\
</code></pre><p>If the same script is run with lines 3 and 12 removed, the following output is generated. Because function <code>afunc</code> and the block-expression are in different MAXScript scope contexts, variable <code>b</code> is implicitly declared as local in each and contain different values. However, because they were implicitly defined, they are not included in the error call stack trace-back.</p>\n\
<p><strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">afunc()\n\
-- Error occurred in j loop\n\
-- Frame:\n\
-- k: undefined\n\
-- j: 1\n\
-- called in i loop\n\
-- Frame:\n\
-- i: 1\n\
-- called in afunc()\n\
-- Frame:\n\
-- No &quot;&quot;*&quot;&quot; function for undefined\n\
OK\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Variables-Assignment-and-Scope.html\">Variables - Assignment and Scope</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Specifying-Global-Variables-As.html\">Specifying Global Variables As Global Using ::</a></li>\n\
          <li><a class=\"link\" href=\"Persistent-Global-Variables.html\">Persistent Global Variables</a></li>\n\
          <li><a class=\"link\" href=\"Type-Free-Variables.html\">Type-Free Variables</a></li>\n\
          <li><a class=\"link\" href=\"Reference-Assignment.html\">Reference Assignment</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Tools-and-Interaction/Creating-MAXScript-Tools/Scripted-Utilities-and-Rollouts/Visibility-of-Locals-Functions.html\">Visibility of Locals, Functions, Structures and User-Interface Items in Rollout Code</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Tools-and-Interaction/Creating-MAXScript-Tools/Scripted-Utilities-and-Rollouts/Accessing-Locals-and-Other-Items.html\">Accessing Locals and Other Items in a Rollout from External Code</a></li>\n\
          <li><a class=\"link\" href=\"Memory-Allocation-and-Garbage.html\">Memory Allocation and Garbage Collection</a></li>\n\
          <li><a class=\"link\" href=\"Memory-Allocation-and-Garbage/Manual-Garbage-Collection.html\">Manual Garbage Collection</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";