var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-C1F6495F-5831-4FC8-A00C-667C5F2EAE36\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>General Event Callback Mechanism</h1></div><p>MAXScript allows you to register callback scripts for all of the notification events supported by <strong>3ds Max</strong>, such as pre/post scene file open, new, reset, scene file save, pre/post render, selection change, etc.</p>\n\
<p>You can specify any number of callback scripts per notification event.</p>\n\
<p>Callback scripts can be bundled into sets with user-defined IDs and can be deleted by type, by ID or by both.</p>\n\
<p>Callback scripts can be specified as persistent so that they are saved and loaded with the currently open file.</p>\n\
<h2 id=\"adding-callbacks\">Adding Callbacks</h2><p>The callbacks are added using the following syntax:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.addScript &lt;callback_type_name&gt; (&lt;function&gt; | &lt;script_string&gt; | &lt;script_stringstream&gt; | fileName:&lt;filename_string&gt;) [id:&lt;name&gt;] [persistent:&lt;boolean&gt;]         </code></pre><p>This method is used to register a new callback script. Requires as the first argument a name that specifies which type of notify event this script is associated with. The list of valid <code>callback_type_name</code> values is listed below.</p>\n\
<p>The script is supplied as a MAXScript function. Any type of function may be supplied: global functions, struct methods, and plug-in methods. Only functions that do not take any parameters, or all parameters have default values, should be registered as callbacks. The <code>addScript()</code> call will fail if you attempt to add a function that takes parameters. Functions passed as callbacks cannot be persisted, and attempting to call with <code>persisted:true</code> will fail.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> When a function is passed to <code>addScript()</code>, it is effectively held as a unique instance. If the function is later re-defined, it does not change the code executed during the callback. To update the function, it needs to be removed and added again.</div>\n\
<p>The script can also be a direct String or StringStream value containing the text of the script to run, or as a <code>fileName:</code> keyword argument, in which case the named file is loaded and run whenever the event notification callback occurs.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p> If the full path of the script is not specified, the script is searched for in the user scripts directory and the 3ds Max system scripts directory.  As of <strong>3ds Max 2022.1 Update</strong>, Windows system directories and directories in the PATH are no longer searched.  See <a href=\"./../File-Access/File-Access-Search.html\"\">File Access Function Search Behavior</a> for a list of directories the <code>filename_string</code> is searched for if the full path is not specified.</p>\n\
\n\
</div></div>\n\
\n\
<p>The optional <code>id:</code> parameter lets you tag one or a group of callbacks with a unique name so that you can remove them all as a group without interfering with other callbacks, perhaps registered by other scripted tools.</p>\n\
<p>The optional <code>persistent:</code> parameter lets you control whether the script is saved permanently in the currently open scene file or is a global callback script that remains in place no matter what file opening and closing is performed. A <code>true</code> value for the parameter specifies that the script should be stored in the current file and loaded and registered for callback whenever that file is opened again. Persistent callback scripts are always removed when a new file is loaded or a reset is performed so that persistent scripts in one file don&apos;t accidentally get copied to a later file. The default for this parameter is <code>false</code> , indicating the script is a global script and is not persistent. Function-based callbacks cannot be persisted, and passing with <code>persistent:true</code> will fail.</p>\n\
<p><strong>EXAMPLES:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.addScript #preRender &quot;setUpRenderGeom()&quot; id:#jbwRender\n\
</code></pre><p>This registers a new callback script that will be called just before a render is performed. The script invokes a function (that has presumably already been set up). A unique ID has been assigned to the script for later selective removal.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- in 3ds Max 2021 and higher\n\
fn myCallback = print &quot;hello&quot;\n\
callbacks.addScript #nodeCreated myCallback id:#myCallbacks</code></pre><p>This registers a new callback that is called whenever a node is created. The callback is a previously defined function.</p>\n\
<h2 id=\"removing-callbacks\">Removing Callbacks</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.removeScripts [&lt;callback_type_name&gt;] [id:&lt;name&gt;]      </code></pre><p>This method is used to unregister and remove one or more callback scripts.</p>\n\
<p>Specifying just a callback event type name removes all the callback scripts for that event type.</p>\n\
<p>Specifying just an <code>id:</code> removes all callback scripts in all events with that ID.</p>\n\
<p>Specifying both limits the ID-based removal to the specified event type.</p>\n\
<p><a name=\"inspecting-callbacks\"></a></p>\n\
<h2 id=\"inspecting-callbacks\">Inspecting Callbacks</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.show [&lt;callback_type_name&gt;] [id:&lt;name&gt;] [to:&lt;stream&gt;] \\\n\
  [asArray: &lt;boolean&gt;] </code></pre><p>This method lists out the current callback scripts or functions in the Listener window, or to an optional stream. Callbacks are listed with their ID, persistent setting, and whether they are loaded from a scene file. If the callback is defined in a string, stringStream or file, the script or filename in which they are defined is also listed. If the callback is defined as a function, the function name, including the struct or plugin it is defined in, is listed.</p>\n\
<p>If the optional callback type name argument is specified in <strong>3ds Max 2009 and higher</strong>, only callbacks of that type will be shown.</p>\n\
<p>If the optional keyword argument <code>id:</code> is specified, only callbacks with that ID will be shown.</p>\n\
<p>If type name and ID are both specified, only callbacks of the type with the given ID will be shown.</p>\n\
<p>Available in <strong>3ds Max 2018.4 and higher</strong>: If the optional keyword argument <code>to:</code> is specified, output from the method is sent to the specified stream.</p>\n\
<p>If the callback definition was loaded from a scene file they are listed as <code>SceneFileEmbedded:true</code>, and considered embedded scripts.  If the Safe Scene Script Execution feature is enabled these types of callbacks are inspected and potentially unsafe commands are blocked from execution.\n\
 <strong>Available in 3ds Max 2021.3 Update and higher.</strong>\n\
If the optional keyword argument <code>asArray:</code> is specified, the output is returned as an array, with an element for each callback. Available in <strong>3ds Max 2019.3 Update and higher</strong>. Each element is an array that contains information about the callback, in this order:</p>\n\
<p><code>(callback type, callback ID, persistent, is filename, script name, scene file embedded)</code></p>\n\
<p><strong>Example</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- show callbacks of type #filePreOpen\n\
callbacks.show #filePreOpen\n\
-- output:\n\
filePreOpen:\n\
  id:#PhysXPlugin, persistent:false, SceneFileEmbedded:false, script:&quot;px_filePreOpen()&quot;\n\
  id:#ALC_SecurityTool, persistent:false, SceneFileEmbedded:false, script:&quot;::ALC_SecurityTool.scenePreOpen()&quot;\n\
  id:#ALC2_SecurityTool, persistent:false, SceneFileEmbedded:false, script:&quot;::ALC2_SecurityTool.scenePreOpen()&quot;\n\
  id:#CRP_SecurityTool, persistent:false, SceneFileEmbedded:false, script:&quot;::CRP_SecurityTool.m_in_file_open = true;::CRP_SecurityTool.register_post_load_check()&quot;\n\
  id:#ADSL_SecurityTool, persistent:false, SceneFileEmbedded:false, script:&quot;::ADSL_SecurityTool.m_in_file_open = true;::ADSL_SecurityTool.register_post_load_check()&quot;\n\
OK\n\
\n\
-- the same thing, but as an array:\n\
callbacks.show #filePreOpen asArray:true\n\
-- output:\n\
#(#(#filePreOpen, #PhysXPlugin, false, false, &quot;px_filePreOpen()&quot;, false), #(#filePreOpen, #ALC_SecurityTool, false, false, &quot;::ALC_SecurityTool.scenePreOpen()&quot;, false), #(#filePreOpen, #ALC2_SecurityTool, false, false, &quot;::ALC2_SecurityTool.scenePreOpen()&quot;, false), #(#filePreOpen, #CRP_SecurityTool, false, false, &quot;::CRP_SecurityTool.m_in_file_open = true;::CRP_SecurityTool.register_post_load_check()&quot;, false), #(#filePreOpen, #ADSL_SecurityTool, false, false, &quot;::ADSL_SecurityTool.m_in_file_open = true;::ADSL_SecurityTool.register_post_load_check()&quot;, false))\n\
\n\
\n\
-- function defined in a struct:\n\
callbacks.show #nodeCreated\n\
-- output:\n\
nodeCreated:\n\
  id:#myCallbacks, persistent:false, SceneFileEmbedded:false, script:&quot;myStruct.myCallback()&quot;\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.broadcastCallback &lt;callback_type_name&gt; </code></pre><p>This method provides a way for you to simulate one of the events and have all the callback scripts for it executed.</p>\n\
<p>Within <strong>3ds Max</strong>, the <code>#preRenderFrame</code> and <code>#postRenderFrame</code> callbacks can only be called by the renderer. These callbacks can not be called by using this method.</p>\n\
<h2 id=\"additional-notification-information-on-the-callback-\">Additional Notification Information on the Callback:</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.notificationParam() </code></pre><p>This method can be called in a callback script to provide additional information on the callback. The value returned by this method varies based on the callback type. If no additional information was present for the callback, a value of &apos;undefined&apos; is returned.</p>\n\
<p><strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">--Remove any existing callbacks with this ID.\n\
callbacks.removeScripts id:#MXSHelp\n\
--Add a new callback script which will call\n\
--callbacks.notificationParam() before selected nodes are deleted\n\
--and thus will print the deleted objects to the Listener:\n\
callbacks.addScript #selectedNodesPreDelete&quot; print (callbacks.notificationParam())&quot; id:#MXSHelp\n\
</code></pre><h2 id=\"callback-event-names-and-notificationparam-return-values\">Callback Event Names and notificationParam Return Values</h2><p>These are all supported the Callback Event Names and the <code>callbacks.notificationParam()</code> values typically returned. Depending on how the broadcast is generated, a value of &apos;undefined&apos; may be returned by this method instead of the value shown.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/ActionItem-Notifications.html\">ActionItem Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Named-Selection-Set-Notification.html\">Named Selection Set Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Animation-Notifications.html\">Animation Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Node-Related-Notifications.html\">Node Related Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Direct3D-Device-Notification.html\">Direct3D Device Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Plug-in-Notifications.html\">Plug-in Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/FileLink-Notifications.html\">FileLink Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Radiosity-Notifications.html\">Radiosity Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/File-Notifications.html\">File Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Rendering-Notifications.html\">Rendering Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Image-Viewer-Display.html\">Image Viewer Display Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Render-Dialog-and-Renderer.html\">Render Dialog and Renderer Plugin Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Layer-System-Notifications.html\">Layer System Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Schematic-View-Notifications.html\">Schematic Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Manipulate-Mode-Notifications.html\">Manipulate Mode Notification</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/System-Notifications.html\">System Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Modifier-Panel-Notifications.html\">Modifier Panel Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Undo-System-Notifications.html\">Undo System Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Modifier-Notifications.html\">Modifier Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Xrefs-Notifications.html\">Xrefs Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Material-Library-Notifications.html\">Material Library Notifications</a></td>\n\
<td><a href=\"General-Event-Callback-Mechanism/Other-Notifications.html\">Other Notifications</a></td>\n\
</tr>\n\
<tr>\n\
<td><a href=\"General-Event-Callback-Mechanism/Material-Notifications.html\">Material Notifications</a></td>\n\
<td></td>\n\
</tr>\n\
</tbody></table>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/ActionItem-Notifications.html\">ActionItem Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Animation-Notifications.html\">Animation Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Direct3D-Device-Notification.html\">Direct3D Device Notification</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/FileLink-Notifications.html\">FileLink Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/File-Notifications.html\">File Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Image-Viewer-Display.html\">Image Viewer Display Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Layer-System-Notifications.html\">Layer System Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Manipulate-Mode-Notifications.html\">Manipulate Mode Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Modifier-Panel-Notifications.html\">Modifier Panel Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Modifier-Notifications.html\">Modifier Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Material-Library-Notifications.html\">Material Library Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Material-Notifications.html\">Material Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Named-Selection-Set-Notification.html\">Named Selection Set Notification</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Node-Related-Notifications.html\">Node Related Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Plug-in-Notifications.html\">Plug-in Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Radiosity-Notifications.html\">Radiosity Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Rendering-Notifications.html\">Rendering Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Render-Dialog-and-Renderer.html\">Render Dialog and Renderer Plugin Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Schematic-View-Notifications.html\">Schematic View Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/System-Notifications.html\">System Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Undo-System-Notifications.html\">Undo System Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Xrefs-Notifications.html\">Xrefs Notifications</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"General-Event-Callback-Mechanism/Other-Notifications.html\">Other Notifications</a></strong></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../../How-To-Practical-Examples/How-To-Change-Objects-At-Render.html\">How To ... Change Objects At Render Time</a></li>\n\
          <li><a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a></li>\n\
          <li><a class=\"link\" href=\"Change-Handlers-and-When.html\">Change Handlers and When Constructs</a></li>\n\
          <li><a class=\"link\" href=\"Time-Change-Callback-Mechanism.html\">Time Change Callback Mechanism</a></li>\n\
          <li><a class=\"link\" href=\"Viewport-Redraw-Callback.html\">Viewport Redraw Callback Mechanism</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";