var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-1961F536-7F01-4D72-930C-4B9432952B79\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Viewport Redraw Callback Mechanism</h1></div><p>You can register one or more functions to be called whenever the <strong>3ds Max</strong> viewports are redrawn.</p>\n\
<p>In addition, with the introduction of the Nitrous Graphics Manager in <strong>3ds Max 2012</strong>, any MAXScript Graphics Window (gw.) <a href=\"./../Interacting-with-the-3ds-Max/Viewports/Viewport-Drawing-Methods.html\">Viewport Drawing Methods</a> must be wrapped in a Viewport Redraw Callback function to produce any results due to the progressive refinement nature of the drawing process.</p>\n\
<p>The following methods let you register and unregister these callbacks:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">registerRedrawViewsCallback &lt;fn&gt;\n\
unRegisterRedrawViewsCallback &lt;fn&gt; ( &lt;fn&gt; | undefined )</code></pre><p>You can register as many functions as you like. Each one is individually called whenever the viewports are redrawn. The functions you register must take no arguments.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn redrawviews_p = print &quot;Viewports Redrawn&quot;\n\
   registerRedrawViewsCallback redrawviews_p\n\
</code></pre><p>   In the above example, the registered function causes the string &quot;Viewports Redrawn&quot; to be printed in the Listener window whenever the <strong>3ds Max</strong> viewports are redrawn.</p>\n\
</blockquote>\n\
<p>You can also enable or disable viewport redraw callbacks globally.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt; redrawViewsCallbacksEnabled()</code></pre><p>Returns whether viewport redraw callbacks are enabled.\n\
 <strong>Available in 3ds Max 2021.1 Update and higher.</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt; enableRedrawViewsCallbacks()\n\
&lt;boolean&gt; disableRedrawViewsCallbacks()</code></pre><p>Enables or disables viewport redraw callbacks. These functions return the current time change callbacks enabled state. So, for example, calling <code>disableRedrawViewsCallbacks()</code> when callbacks are already disabled will return <strong>false</strong>.\n\
 <strong>Available in 3ds Max 2021.1 Update and higher.</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\"> showregisteredRedrawViewsCallbacks [to:stream] [asArray:&lt;boolean&gt;]</code></pre><p>Returns a list of registered viewport redraw callbacks, listing the function name, source file (if applicable) and line number in the source file where the function is defined. If the optional <strong>to</strong> argument is specified, the output is sent to that output stream. If the optional <strong>asArray</strong> argument is specified, the return value is formatted as an array of arrays, where for each registered callback function the first element is the callback function name, the second element is the source file name and line number where the callback function is defined (or an empty string if the function was not defined in a file), and the third element is the actual function value which can be used to de-register the callback.</p>\n\
<p><strong>Special Considerations</strong></p>\n\
<ul>\n\
<li><p>If a runtime error occurs in a callback function while it is being executed, an error message is displayed and that callback function is permanently disabled.</p>\n\
</li>\n\
<li><p>The registered function is executed in a special context and not in the context of the code that created it. This means that the function cannot contain references to local variables in outer code nestings surrounding the function definition since those variables are on an execution stack that does not exist at the time the function is called. An important exception to this is utility and rollout panel locals, such as local functions, rollout variables and nested rollouts. You can refer to them in change handler code inside rollout code as they are associated directly with their rollout or utility object.</p>\n\
</li>\n\
<li><p>Note that it is the <strong>function value</strong> that is being registered, not the function name or global variable. This means that redefining the same-named function after registering it will not change the callback automatically, but register another function value to be called resulting in two separate callbacks. You either need to unregister the old callback function, then redefine the function and register it again (see further on this page), or make the registered function an intermediary which calls another function,</p>\n\
</li>\n\
</ul>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn redrawviews_cb = print currentTime\n\
   fn rvcb = redrawviews_cb()\n\
   registerRedrawViewsCallback rvcb\n\
</code></pre><p>   In this case, the registered callback function, <code>rvcb</code> , calls the real call back, <code>redrawviews_cb</code> , (by referencing the global variable it is defined in), meaning you can redefine <code>redrawviews_cb()</code> as often as you need and the callback will always invoke the latest definition. This is a useful technique to employ while you are developing and debugging a callback.</p>\n\
<p>   Note that you may only redefine the <code>redrawviews_cb()</code> function but NOT evaluate the other two lines again, otherwise you would end up with multiple callbacks calling the same function on each redraw.</p>\n\
</blockquote>\n\
<ul>\n\
<li>Due to the fact that the function value is being registered, it is absolutely important to unregister any previous value using the <code>unregisterRedrawViewsCallback()</code> method before redefining the function and registering a new callback.The method can be called with an undefined argument, so it can be safely placed just before the function definition without the need to ensure the function already exists. This will ensure that a previous definition of the function will not remain registered without any means to unregister it (short of restarting <strong>3ds Max</strong>).</li>\n\
</ul>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   unregisterRedrawViewsCallback redrawviews_callback_function\n\
   fn redrawviews_callback_function = print &quot;Viewport Redraw 1&quot;\n\
   registerRedrawViewsCallbackredrawviews_callback_function\n\
</code></pre><p>   Evaluating the above will first attempt to unregister the callback function. If it was not registered, nothing will happen. Then the function will be defined and its value will be registered as callback. Orbiting in the viewport for example will cause the string to be printed to the listener.</p>\n\
<p>   If you would change the string to &quot;Viewport Redraw 2&quot; and evaluate the same code, the old function will be unregistered, the new function will be defined and the new callback will be registered. Orbiting the viewport will print the new string only.</p>\n\
<p>   <strong>BAD EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn redrawviews_callback_function = print &quot;Viewport Redraw2&quot;\n\
   unregisterRedrawViewsCallback redrawviews_callback_function\n\
   registerRedrawViewsCallback redrawviews_callback_function\n\
</code></pre><p>   If the unregister function call were the second line and not the first, the new function would be defined first, then the unregister attempt would fail silently because the new function would be used and the old one&apos;s value would be inaccessible. This would result in both the old and the new functions being executed on viewport redraws, thus printing both &quot;Viewport Redraw 1&quot; and &quot;Viewport Redraw 2&quot; to the Listener.</p>\n\
<p>   <strong>LOCAL SCOPE EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (--open a local scope\n\
   global redrawviews_cb_function --ensure variable visibility\n\
   unregisterRedrawViewsCallback redrawviews_cb_function\n\
   fn redrawviews_cb_function= print &quot;Redraw Test1&quot;\n\
   registerRedrawViewsCallback redrawviews_cb_function\n\
   )--end local scope\n\
</code></pre><p>   If the function definition and the callback unregistering and registering are performed in local scope, it is paramount to ensure the callback function&apos;s visibility to the code in either global scope or a &quot;private local&quot; scope such as the top-level local scope of a MacroScript that would protect the function value during the current session. Evaluating the above code without a global declaration would create a function value inside the LOCAL scope defined by the outer-most brackets and the attempt to call <code>unregisterRedrawViewsCallback()</code> in a second evaluation of the script would NOT see the original function value in the variable since the variable would have ceased to exist after the first run. Thus, the variable storing the function value must be declared as global to allow a second run of the script to successfully unregister the previous function value.</p>\n\
<p>   Changing the printed string to &quot;Redraw Test 2&quot; and evaluating the code again would correctly unregister the old value stored in the global variable, then overwrite it with the new value and register it as a redraw views callback.</p>\n\
</blockquote>\n\
<ul>\n\
<li>Callback functions remain registered after loading a new file or performing a <strong>3ds Max</strong> reset.</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a></li>\n\
          <li><a class=\"link\" href=\"Change-Handlers-and-When.html\">Change Handlers and When Constructs</a></li>\n\
          <li><a class=\"link\" href=\"Time-Change-Callback-Mechanism.html\">Time Change Callback Mechanism</a></li>\n\
          <li><a class=\"link\" href=\"General-Event-Callback-Mechanism.html\">General Event Callback Mechanism</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";