var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-619AF4D3-A347-4155-943B-707D421BC460\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Mouse Tool Clauses</h1></div><p>The <code>&lt;tool_body&gt;</code> of a mouse tool definition is made up of a sequence of tool clauses as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;tool_body&gt; ::= { &lt;tool_clause&gt; }+</code></pre><p>Tool clauses define the components of a tool and can be one of two basic things:</p>\n\
<ul>\n\
<li><p><strong>Local variables, functions or structure definitions</strong> are variables, functions, and structures whose scope is the tool. The <code>&lt;tool_body&gt;</code> is compiled in its own local scope and the locals are only visible inside the scope of the tool. Tool locals are heap-based locals, which are slightly different from normal (stack-based) locals. Normal locals are visible in the current scope and have a lifetime of one execution of that scope. Heap-based locals are also visible only in the current scope, but have a lifetime equal to the lifetime of the top-level expression where they are defined. A tool&#x2019;s locals are created the first time you execute the tool and are kept permanently in the heap (unless and until you redefine the tool). This means things like rollouts created in a tool that live beyond the simple execution of the tool can refer to these locals and the locals will still exist. Each time you execute a tool, its local variables are initialized to a value of <code>undefined</code> , or to their specified initialization value. For example, you can create a rollout in a tool, and the rollout&#x2019;s event handlers can access the locals defined in the tool because the rollout is executing within the scope of the tool. You cannot access the tool&#x2019;s locals from another utility or from Listener, because they are not executing within the scope of the tool. See <a href=\"./../../../MAXScript-Language-Reference/Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a> for more information.</p>\n\
</li>\n\
<li><p><strong>Event handlers</strong> are special kinds of functions associated with particular events. Event handlers specify the processing you want to occur when a user clicks or moves a mouse, right-clicks, or starts or stops the tool. These actions generate named events and the optional event handler you supply for that event is called when the action occurs.</p>\n\
</li>\n\
</ul>\n\
<p>Formally, the syntax of a <code>&lt;tool_clause&gt;</code> is defined as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;tool_clause&gt; ::= &lt;local_variable_decl&gt; | &lt;local_function_decl&gt; | &lt;local_struct_decl&gt; | &lt;event_handler&gt;</code></pre><p><strong>Locals</strong></p>\n\
<p>A <code>&lt;local_variable_decl&gt;</code> , <code>&lt;local_function_decl&gt;</code> , and <code>&lt;local_struct_decl&gt;</code> are exactly the same as local variable, function, and structure definitions in MAXScript:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_variable_decl&gt; ::= local &lt;decl&gt; { , &lt;decl&gt; } \n\
&lt;decl&gt; ::= &lt;name&gt; [ = &lt;expr&gt; ] -- optional initial value\n\
&lt;local_function_decl&gt; ::= [ mapped ](function | fn ) &lt;name&gt; { &lt;argument&gt; } = &lt;expr&gt;\n\
&lt;local_struct_decl&gt; ::+ struct &lt;name&gt; ( &lt;member&gt; { , &lt;member&gt; } )\n\
&lt;member&gt; ::= ( &lt;name&gt; [ = &lt;expr&gt; ] | &lt;local_function_decl&gt; )</code></pre><p>Global variables cannot be declared as a tool clause, however they can be declared within event handler scripts. If you need to ensure a variable name references a global variable, declare the variable name as global immediately before defining the tool in your script.</p>\n\
<p>When writing scripts, it is good programming practice to explicitly declare your local and global variables. Implicit declaration is provided as a short-hand, typically used when working in the Listener interactively or developing short scripts. When developing extended scripts, explicitly declaring variables can reduce errors and improve readability of the code. It is also recommend that you declare as local all variables unless you really want them to be global variables. The reasons for this are described in <a href=\"./../../../MAXScript-Language-Reference/Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a>.</p>\n\
<p>Within the functions and event handlers of a tool, 13 pre-defined locals variables are always accessible. Each one holds a useful value relating to the mouse action that just occurred:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">viewPoint Point2</code></pre><p>The current mouse position in viewport pixel coordinates.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">worldPoint Point3</code></pre><p>The current mouse position projected on the active grid in world coordinates.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">worldDist Point3</code></pre><p>The distance in X, Y, and Z from the previous click point to the current click point in world coordinates.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">worldAngle Point3</code></pre><p>The angles around the world X, Y, and Z axes from the previous click point to the current click point in world coordinates.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gridPoint Point3</code></pre><p>The current mouse position projected on the active grid in the coordinate system of the active grid. If the active grid is the home grid, the coordinate system is the home grid plane active in the current viewport (that is, the construction plane). See the discussion of grid versus world values below.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gridDist Point3</code></pre><p>The distance in X, Y, and Z from the previous click point to the current click point in the coordinate system of the active grid. If the active grid is the home grid, the coordinate system is the home grid plane active in the current viewport (that is, the construction plane). See the discussion of grid versus world values below.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gridAngle Point3</code></pre><p>The angles around the world X, Y, and Z axes from the previous click point to the current click point in the coordinate system of the active grid. If the active grid is the home grid, the coordinate system is the home grid plane active in the current viewport (that is, the construction plane). See the discussion of grid versus world values below.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">shiftKey Boolean</code></pre><p><code>true</code> if <strong>SHIFT</strong> key was down.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">ctrlKey Boolean</code></pre><p><code>true</code> if <strong>Ctrl</strong> key was down.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">altKey Boolean</code></pre><p><code>true</code> if <strong>Alt</strong> key was down.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">lButton Boolean</code></pre><p><code>true</code> if left mouse button was down.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">mButton Boolean</code></pre><p><code>true</code> if middle mouse button was down.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">rButton Boolean</code></pre><p><code>true</code> if right mouse button was down.</p>\n\
<p>When a create mouse tool is used in a level 5 scripted plug-in, an additional local variable is accessible:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">nodeTM Matrix3</code></pre><p>Provides read/write access to the transform of the node currently being created. This value is in the current grid coordinate system.</p>\n\
<p>See <a href=\"./../Scripted-Plug-ins/Scripted-Plug-ins-1.html\">Scripted Plug-ins</a> for more information.</p>\n\
<p>When writing SimpleObject scripted plug-ins, you should always use the <code>gridPoint</code> , <code>gridDist</code> , and <code>gridAngle</code> values rather than corresponding world values, as object-creation in SimpleObject scripted plug-ins is managed in the active grid coordinate system.</p>\n\
<p>For <code>gridDist</code> , the .X and .Y components are the delta X and Y between the previous clicked point and the current mouse point in the plane of the current grid. The .Z component is a projection from the current Y screen coordinates onto a Z-vector (in the grid coordinate system) based at the last point clicked on the grid, such that the .Z value is the projected height up the Z-vector. For non-orthogonal viewports this is as expected, but for orthogonal viewports (in which you are always dragging in the XY plane of the grid), this is always the same as <code>gridDist.y</code> . If you are using the gridDist value to build the portion of an object on the grid, or determine the distance on the current grid from the last point, you want to use only the .X and .Y components.</p>\n\
<p><strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">side1Len = gridDist.x\n\
side2Len = gridDist.y\n\
dist=length [gridDist.x,gridDist.y]\n\
</code></pre><p>If you are specifying the height off of the current grid, you would typically use the .Z component.</p>\n\
<p><strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">height = gridDist.z\n\
</code></pre><p>For <code>worldDist</code> , the behavior is similar to that for <code>gridDist</code> , however the projected component is dependent on the construction plane of the viewport. For the Top, Bottom, and non-orthogonal viewports, the projected height is contained in the .Z component. For Left and Right viewports, the projected height is contained in the .X component. For Front and Back viewports, the projected height is contained in the .Y component.</p>\n\
<p>When you create a node in the <strong>3ds Max</strong> user-interface, the local Z axis of the node is perpendicular to the construction plane. When you create a node using MAXScript, by default the local Z axis of the node points along the direction of the world Z axis. If you create a node in a tool (other than in a SimpleObject scripted plug-in), you must take into account the construction plane orientation if you want to duplicate <strong>3ds Max</strong>&#x2019;s behavior when creating a node. The easiest way to do this is to create the node in a <code>coordsys grid</code> context and specify the position of the node during creation in grid coordinate space. An example of this is shown in the following script.</p>\n\
<p><strong>POINT CREATOR</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">tool PointCreator\n\
(\n\
local p, createpoint\n\
-- define a function to perform actual node creation. Setting coordsys to\n\
-- &#x2019;r;grid&#x2019; in order for the alignment of the node&#x2019;s local Z axis to be\n\
-- perpendicular to the construction grid\n\
fn createpoint = in coordsys grid p=point pos:gridPoint\n\
--\n\
-- set up so that a node is created on a mouse button down, move node\n\
-- drag, release node at mouse button up.\n\
--\n\
-- if clickno == 1, then we are at first mouse click, which is a mouse\n\
-- button down. If clickno != 1, at following mouse button up.\n\
on mousePoint clickno do\n\
(\n\
if clickno == 1\n\
then createPoint()\n\
-- if p == undefined, then clicked twice without mouse movement\n\
-- (double clicked). No point object present, so just ignore this click.\n\
else if p != undefined do (p.pos=worldPoint;p=undefined)\n\
)\n\
-- if p != undefined, we are moving a previously created node\n\
-- if p == undefined, and left mouse button is down, create a node\n\
on mouseMove clickno do\n\
(\n\
if p != undefined\n\
then p.pos=worldPoint\n\
else if lbutton do createPoint()\n\
)\n\
)\n\
-- start the tool. No exit condition defined, so right-click to exit.\n\
startTool PointCreator\n\
</code></pre><p><strong>Event Handlers</strong></p>\n\
<p>An <code>&lt;event_handler&gt;</code> is a special function definition local to a tool that you provide to handle the processing you want to occur when a user clicks in a viewport, right-clicks, or starts or stops the tool. Each user action generates a named event and any event handler you supply for that event is called when the action occurs. The available event handlers are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on start do &lt;expr&gt;</code></pre><p>Called when the tool starts.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on stop do &lt;expr&gt;</code></pre><p>Called when the tool ends.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on freeMove do &lt;expr&gt;</code></pre><p>Called when the mouse moves prior to the first click.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on mousePoint &lt;arg&gt; do &lt;expr&gt;</code></pre><p>Called for each mouse click, parameter <code>arg</code> contains the number of the click.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on mouseMove &lt;arg&gt; do &lt;expr&gt;</code></pre><p>Called when the mouse moves anytime after the first click.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on mouseAbort &lt;arg&gt; do &lt;expr&gt;</code></pre><p>Called when the user right-clicks to cancel or presses the ESC key.</p>\n\
<p>The <code>&lt;arg&gt;</code> parameter on each of the mouse event handlers lets you know which mouse click you are in. Mouse clicks are actually only the mouse click release, except for the initial mouse click. If you do a mouse click, drag, release, move, click, drag, and release, the <code>on mousePoint</code> event handler is called three times: the first click, the first release, and the second release, with <code>&lt;arg&gt;</code> values of 1, 2, and 3, respectively. The mouse click counter is incremented after processing the <code>on mousePoint</code> event handler, so the <code>&lt;arg&gt;</code> parameter for the <code>mouseMove</code> and <code>mouseAbort</code> event handlers specify the click count for the next mouse click. In the previous example, when you drag between the first click and first release, the <code>&lt;arg&gt;</code> value for a <code>mouseMove</code> event handler will have a value of 2. During the following move, click, and drag it will have a value of 3.</p>\n\
<p>In <strong>3ds Max 6 and higher,</strong> while running a mouse tool, if you click in a viewport where the construction plane is on edge, the tool will exit with a return value of <code>#abort</code></p>\n\
<p>The following script simply demonstrates when the various event handlers are called. Simply run this script and drag in the viewports. Right-click or press the ESC key to stop the tool&#x2019;s execution.</p>\n\
<p><strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">tool foo\n\
(\n\
on freeMove do print &quot;Free Moving&quot;\n\
on mousePoint clickno do format &quot;Point: %\\n&quot; clickno\n\
on mouseAbort clickno do format &quot;Abort: %\\n&quot; clickno\n\
on mouseMove clickno do format &quot;Moving: %\\n&quot; clickno\n\
on start do print &quot;Starting&quot;\n\
)\n\
startTool foo prompt:&quot;Hello!&quot;\n\
</code></pre><p>All of the mouse action event handlers may return the special value <code>#stop</code> which causes the tool to stop. In the following <code>mousePoint</code> event handler example, the first click creates the box and the release (second click) terminates the tool.</p>\n\
<p><strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on mousePoint clickno do\n\
if clickno == 1\n\
then b = box pos:worldPoint\n\
else #stop\n\
</code></pre><p>The following script implements the following functions: on mouse click three spot lights are created at the mouse point. While holding the mouse button down, drag the mouse to position the classical key, back and fill light positions. During this drag, holding down the SHIFT key will flip the fill light side. Release the mouse button and move the mouse to change the elevation of the lights. Click the mouse button when the lights are at the correct height. The back light is raised 1.5 times the key light&#x2019;s height, and the fill light is raised 0.75 times the key light&#x2019;s height.</p>\n\
<p><strong>TO START THIS TOOL, YOU WOULD SAY</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">startTool three_lights\n\
</code></pre><p>and then drag in one of the viewports.</p>\n\
<p><strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">tool three_lights\n\
(\n\
local key, fill, back, targ\n\
on mousePoint click do coordsys grid\n\
(\n\
if click == 1 then -- create key, back &amp; fill lights at mousedown\n\
(\n\
targ = targetobject pos:gridPoint\n\
key = targetspot pos:gridPoint name:&quot;key&quot;target:targ\n\
back = targetspot pos:gridPoint name:&quot;back&quot;target:targ\n\
fill = targetspot pos:gridPoint name:&quot;fill&quot;target:targ\n\
)\n\
if click == 3 then #stop\n\
)\n\
on mouseMove click do\n\
(\n\
if click == 2 then -- drag out &amp; round on x-y plane\n\
(\n\
coordsys grid key.pos = gridPoint\n\
coordsys targ back.pos = - key.pos\n\
local p = if shiftKey then 90 else -90\n\
coordsys targ fill.pos = key.pos * ((eulerangles 0 0 p) as quat)\n\
)\n\
else if click == 3 then -- drag up to elevate lights\n\
(\n\
in coordsys targ\n\
(\n\
local Z = gridDist.z\n\
key.pos.z = Z\n\
back.pos.z = Z * 1.5\n\
fill.pos.z = Z * 0.5\n\
)\n\
)\n\
)\n\
)\n\
</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Mouse-Tools.html\">Scripted Mouse Tools</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Scripted-Mouse-Tools.html\">Scripted Mouse Tools</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";