var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-AEE0D822-85A0-4F90-8B3C-C6336D8694B4\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>MAXtoA Scripted Plug-ins</h1></div><p>Available in <strong>3ds Max 2018 and higher</strong>: MAXScript provides a simple way to create a &quot;wrapper&quot; plug-in with a custom UI for Arnold nodes. This approach is for plug-ins that are not shaders - shader plug-ins use a different approach.</p>\n\
<p>A MAXtoA plug-in uses specially-named parameters that the MAXtoA translator understands. MAXtoA ignores other parameters and the internals of the plug-in, whether it contains logic or wraps another existing 3ds Max class. 3ds Max sees the wrapped class and plug-in logic, which will show up in the UI (viewport, modifier stack, etc depending on plug-in type), and Arnold (via MAXtoA) reacts only to the special parameters, using them to instantiate some particular Arnold node and set parameters on it accordingly.</p>\n\
<p>The following plug-in types can be extended:</p>\n\
<ul>\n\
<li>Geometry - Any scripted geometry plugin can be turned into any Arnold shape node.</li>\n\
<li>Cameras - A scripted camera can be turned into any Arnold camera node.</li>\n\
<li>Modifiers on Lights - A scripted modifier applied to an Arnold Light can be turned into any Arnold light filter node.</li>\n\
</ul>\n\
<h2 id=\"parameter-layout-and-naming\">Parameter Layout and Naming</h2><p>MAXtoA looks for exposed parameters that following a specific naming convention, and acts on those parameters accordingly. The parameter names that MAXtoA recognizes are:</p>\n\
<ul>\n\
<li><strong>arnold_node</strong>: This parameter must be of type <strong>#string</strong> and contains the name of the Arnold node to be instantiated for this particular script.</li>\n\
<li><strong>arnold_node_xxx</strong>: This is the way to set parameter named <em>xxx</em> on the Arnold node. That is, anything following the prefix &quot;arnold_node_&quot; is taken to be the parameter name. The parameter type in the scripted plug-in must match what the Arnold node expects.</li>\n\
<li><strong>arnold_link*xxx</strong>: This is the way to set parameter <em>xxx</em> as a link to a shader on the Arnold node. Anything following the prefix &quot;arnold_link_&quot; is taken to be the parameter name on the Arnold node. If the value of the parameter (which must be of type <strong>#texmap</strong>) is non-empty, the output of that texmap (or rather the Arnold shader implementing that texmap) will be connected to the Arnold node&apos;s input, and the value will come from it, replacing the value given by any <strong>arnold_node*xxx</strong> parameter, if present.</li>\n\
<li><strong>arnold_decl*xxx</strong>: This declares user-defined parameters before they are used. Anything following the &quot;arnold_decl_&quot; prefix is taken to be the parameter name. The parameter itself must be of type <strong>#string</strong> and contain an Arnold parameter declaration string, for example &quot;constant FLOAT&quot; for a float value.</li>\n\
</ul>\n\
<p>Parameters are processed in the order they appear in the script, from top to bottom. Declarations must precede usage.</p>\n\
<p><strong>Enum Parameters</strong></p>\n\
<p>Arnold parameters of type enum are normally represented by and integer value starting at zero. However, MAXScript dropdown controls are one-based. Therefore, MAXtoA translates enum type parameters to one-based arrays to support dropdowns in plug-in UIs. This snippet illustrates how enums are defined and wired to a UI control.</p>\n\
<h3 id=\"enum-snippet\">Enum Snippet</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">parameters main rollout:params\n\
(\n\
   . . .\n\
   arnold_node_mode type:#integer default:2 ui:mode\n\
   . . .\n\
)\n\
\n\
rollout params &quot; Parameters&quot;\n\
(\n\
   . . .\n\
   dropdownlist mode &quot;Mode&quot; items:#(&quot;Blend&quot;, &quot;Mix&quot;, &quot;Replace&quot;, &quot;Add&quot;, &quot;Sub&quot;)\n\
   . . .\n\
)</code></pre><p>The parameter on the scripted node (the parameter named <strong>arnold_node_mode</strong>) will have values 1, 2, 3, 4 or 5 based on what is selected by the dropdown, however, the value actually sent to Arnold will be 0, 1, 2, 3, or 4.</p>\n\
<h2 id=\"example-a-spherical-camera-plug-in\">Example: A Spherical Camera Plug-in</h2><p>This example creates a new camera type, derived from FreeCamera, named &quot;Spherical Cam&quot; in the Arnold category, which simply turns on the Arnold spherical_camera node. This script sets no other parameters.</p>\n\
<h3 id=\"example\">Example</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">plugin Camera arnold_spherical_camera\n\
extends:FreeCamera\n\
name:&quot;Spherical Cam&quot;\n\
classID:#(135251,542126)\n\
category:&quot;Arnold&quot;\n\
replaceUI:true\n\
(\n\
parameters main rollout:params\n\
    (\n\
        arnold_node type:#string default:&quot;spherical_camera&quot;\n\
    )\n\
\n\
    rollout params &quot;Example Parameters&quot;\n\
    (\n\
\n\
    )\n\
)</code></pre><h2 id=\"example-sphere-geometry-plug-in\">Example: Sphere Geometry Plug-in</h2><p>This example creates a new geometry type in the Arnold category called &quot;Sphere&quot;. It extends the built-in 3ds Max sphere, so it shows up in the viewport as such. But Arnold will actually be given the mathematically exact &quot;sphere&quot; shape node when rendering.</p>\n\
<p>Most of the code in this example enables geometry creation with the mouse and to keeping that in sync with the parameter defining the Arnold object. A much simpler piece of code could have been written had we completely ignored the wrapped object; we could have set the UI in the rollout to apply directly to the Arnold style parameters, which means no <code>updateArnold()</code> function would have been needed, and it could have been only a few lines.</p>\n\
<h3 id=\"example\">Example</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">plugin Geometry Arnold_Sphere\n\
name:&quot;Sphere&quot;\n\
classID:#(0xf00df103, 0xc55744a1)\n\
category:&quot;Arnold&quot;\n\
extends:Sphere\n\
replaceUI:true\n\
(\n\
    local lastSize, meshObj\n\
    parameters pblock rollout:params\n\
    (\n\
        arnold_node type:#string default:&quot;sphere&quot;\n\
        arnold_node_radius type:#float default:1.0\n\
        arnold_node_center type:#point3 default:[0.0,0.0,0.0]\n\
\n\
        radius type:#float animatable:true ui:radius default:1.0\n\
    )\n\
\n\
    fn fmax val1 val2 = if val1 &gt; val2 then val1 else val2\n\
\n\
    fn updateArnold =\n\
    (\n\
          -- Modify the Arnold Sphere\n\
          arnold_node_radius = delegate.radius\n\
    )\n\
\n\
    tool create\n\
    (\n\
        on mousePoint click do\n\
        case click of\n\
        (\n\
            1: nodeTM.translation = gridPoint\n\
            2: #stop\n\
        )\n\
        on mouseMove click do\n\
        (\n\
            if click == 2 then (            \n\
                radius = delegate.radius = fmax (abs gridDist.x) (abs gridDist.y)\n\
                updateArnold()\n\
            )\n\
        )\n\
    )  \n\
\n\
    rollout params &quot;Sphere Parameters&quot;\n\
    (\n\
        Spinner radius &quot;Radius:&quot; range:[0, 1e9, 1]\n\
\n\
        on radius changed val do (\n\
            delegate.radius = val\n\
            updateArnold()\n\
        )\n\
    )\n\
)</code></pre><p>The key part of this code is the Arnold-specific parameter declarations:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">arnold_node type:#string default:&quot;sphere&quot;\n\
arnold_node_radius type:#float default:1.0\n\
arnold_node_center type:#point3 default:[0.0,0.0,0.0]</code></pre><p>These parameters tell Arnold to build a node named &#x2018;sphere&#x2019; and assign its &#x2018;radius&#x2019; parameter of type float and the &#x2018;center&#x2019; parameter of type point with the value from these variables.</p>\n\
<p>Had we needed custom declared user variables (which are needed, for example, in the volume node), it would have looked like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">arnold_node type:#string default:&quot;volume&quot;\n\
arnold_node_min type:#point3 default:[-0.5,-0.5,0] \n\
arnold_node_max type:#point3 default:[0.5,0.5,1] \n\
arnold_node_dso type:#string default:&quot;volume_openvdb&quot;\n\
\n\
arnold_decl_filename type:#string default:&quot;constant STRING&quot;\n\
arnold_node_filename type:#filename default:&quot;smoke.vdb&quot;\n\
\n\
arnold_decl_bounds_slack type:#string default:&quot;constant FLOAT&quot;\n\
arnold_node_bounds_slack type:#float default:0.0</code></pre><p>This would instantiate a node called volume as before, and set its min, max and dso parameters. But the rest of the parameters (due to the plug-in nature of volumes) are user defined, so we need to define a string parameter called &quot;filename&quot;, a float parameter called &quot;bounds_slack&quot; and so on and so on.</p>\n\
<h2 id=\"example-gobo-filter-plug-in\">Example: Gobo Filter Plug-in</h2><p>This example is a stripped down version of the Arnold Gobo filter modifier plugin (located in &lt;3dsmax&gt;\\Plugins\\maxscript\\arnoldlight_gobofilter.ms). It show case how you can add UI controls to support both a color picker and a Texmap lookup. The Arnold target parameter here is gobo.slidemap.</p>\n\
<h3 id=\"example\">Example</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">plugin modifier Arnold_Light_Gobo\n\
name:~ARNOLD_LIGHT_FILTER_GOBO_NAME~\n\
category:&quot;Arnold&quot;\n\
\n\
classID:#(0xf00df10d,0x4001a10d)\n\
\n\
extends:ArnoldLightFilterModifier replaceUI:true version:1\n\
(\n\
  parameters main rollout:params\n\
  (\n\
    arnold_node type:#string default:&quot;gobo&quot;  \n\
arnold_node_slidemap type:#color ui:slidemapcolor default:[255,255,255]\n\
    arnold_link_slidemap type:#texturemap ui:slidemap\n\
  )\n\
  rollout params ~ARNOLD_LIGHT_FILTER_GOBO_ROLLOUT_NAME~\n\
  (\n\
      colorpicker slidemapcolor &quot;Color&quot; color:[255,255,255] modal:false\n\
      mapButton slidemap &quot;Map&quot;\n\
  )\n\
)</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Plug-ins.html\">Scripted Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";