var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-2662358C-43AA-460D-9B0F-C53CC91F769C\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Scripted Camera Plug-ins</h1></div><p>Scripted camera plug-ins can be written in MAXScript. At present, you can write either extending camera plug-ins that extend the existing MAX cameras or temporary camera plug-ins that can be used to host create tools for a custom camera system.</p>\n\
<p>The plug-in superclass for cameras is &apos;camera&apos;.</p>\n\
<p><strong>Event Handlers</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on renderApertureChanged val do ....</code></pre><p>This event handler is available to scripted cameras and gives access to the CameraObject:: <code>renderApertureChanged</code> callbacks made by the rendering dialog to a camera when the user adjusts parameters that affect the render aperture.</p>\n\
<p>This handler is optional.</p>\n\
<p>The <code>&apos;val&apos;</code> parameter contains the new render aperture value. This typically gets called when you select a new aperture in the Output Size dropdown in the render dialog or adjust a spinner when in Custom mode.</p>\n\
<p>There are several cases where this handler is called:</p>\n\
<ul>\n\
<li><p>In the Render Scene dialog, if Image Aspect is unlocked and you change the Width, Height, or Pixel Aspect</p>\n\
</li>\n\
<li><p>In the Render Scene dialog, if you click on one of the output size preset buttons</p>\n\
</li>\n\
<li><p>In the Render Scene dialog, if Image Aspect is locked and you change the Width, Height, or Pixel Aspect, or if you change Aperture Width, and then click on either Render or Close</p>\n\
</li>\n\
</ul>\n\
<p>The value being passed into handler is the aperture width. You can get the remaining values using the <code>renderWidth</code> , <code>renderHeight</code> , and <code>RenderPixelAspect</code> global variables. However, what you get for these values are typically the <strong>old</strong> value with the exception of the Aperture Width value. The call to the handler is coming from a &quot;InvalidateCameras&quot; method, which is called right before setting the new values. No broadcasts are made after the new values are set, and if safe frames are not turned on in any viewport, none of the viewports are redrawn.</p>\n\
<p>It is recommended that if a scripted camera uses this handler, it must cache the old Aperture Width value and test the incoming value against the cached value before proceeding.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   plugin Camera CamTest\n\
   name:&quot;CamTest&quot;\n\
   classID:#(0x47db14ff, 0x4e9b5f90)\n\
   category:&quot;Standard&quot;\n\
   extends:FreeCamera\n\
   ( \n\
     on renderApertureChanged val do \n\
       format &quot;renderApertureChanged: %\\n&quot; val \n\
     tool create ( on mousePoint click do #stop ) \n\
   ) \n\
</code></pre></blockquote>\n\
<p><strong>The following handler was added in 3ds Max 7:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on getDisplayMesh do ....</code></pre><p>This event handler is available to scripted cameras and <a href=\"Scripted-Helper-Plug-ins.html\">helpers</a>, and lets the developer replace the viewport display mesh of the delegate with a custom display mesh.</p>\n\
<p>This handler is optional.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> For <strong>performance</strong> reasons, it is <strong>critical</strong> to regenerate the returned mesh as little as possible. The handler is called often for display, bounds testing, and hit testing.</div>\n\
<p>The example below shows how to update the mesh in the handler only when the mesh has actually changed:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   plugin Camera CamTest_DisplayMesh\n\
   name:&quot;CamTest&quot;\n\
   classID:#(0x6ca13d13, 0x12afbfd4)\n\
   category:&quot;Standard&quot;\n\
   extends:FreeCamera\n\
   (\n\
     local lastR1, lastR2, meshObj\n\
     parameters pblock rollout:params\n\
     (\n\
       radius1 type:#float animatable:true ui:r1_amount default:40.0\n\
       radius2 type:#float animatable:true ui:r2_amount default:20.0\n\
     )\n\
     rollout params &quot;CamTest Parameters&quot;\n\
     (\n\
       Spinner r1_amount &quot;Radius 1:&quot; range:[0, 1e9, 40]\n\
       Spinner r2_amount &quot;Radius 2:&quot; range:[0, 1e9, 20]\n\
     )\n\
     on getDisplayMesh do\n\
     (\n\
       if (meshObj == undefined) do\n\
       (\n\
         meshObj = createInstance torus radius1:radius1 radius2:radius2 mapCoords:false\n\
         lastR1 = radius1; lastR2 = radius2\n\
       )\n\
       if radius1 != lastR1 do (meshObj.radius1 = radius1; lastR1 = radius1)\n\
       if radius2 != lastR2 do (meshObj.radius2 = radius2; lastR2 = radius2)\n\
       meshObj.mesh\n\
     )--end on getDisplayMesh\n\
     tool create\n\
     (\n\
       on mousePoint click do (nodeTM.translation = gridPoint;#stop )\n\
     )--end tool\n\
   )--end plugin\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on useWireColor do ....</code></pre><p>This <strong>optional</strong> event handler lets the developer specify whether to use the node&apos;s wire color when drawing the object in the viewport, or use the UI color for the object type (Camera Object for Cameras). If this handler returns true, the node&apos;s wire color is used, otherwise the UI color for the object type is used. If the event handler is not specified, the wire color is used. This handler is only applicable when a <code>getDisplayMesh</code> handler is supplied. Available in <strong>3ds Max 8 and higher</strong>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Plug-ins.html\">Scripted Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Scripted-Plug-ins-1.html\">Scripted Plug-ins</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";