var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-1361B847-7878-4C52-98DE-5BC5CA830B40\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Rollout Clauses</h1></div><p>Rollout clauses define the components of a rollout (utilities are considered a special case of rollouts) and can be one of these four basic things:</p>\n\
<ul>\n\
<li><p><strong>Local variables, functions, or structure definitions</strong> are variables, functions, and structures whose scope is the rollout. These locals will exist as long as the rollout value exists. The rollout value will exist until a new value is assigned to the rollout&#x2019;s variable name. Local variables are particularly useful for storing the working data associated with the rollout such as, picked objects or arrays of generated objects. The visibility of locals and accessing rollout locals from external code are described in <a href=\"Visibility-of-Locals-Functions.html\">Visibility of Locals, Functions, Structures, and User-Interface Items in Rollout Code</a> and <a href=\"Accessing-Locals-and-Other-Items.html\">Accessing Locals and Other Items in a Rollout from External Code</a>.</p>\n\
</li>\n\
<li><p><strong>Mouse tools</strong> are used to perform a set of actions based on mouse clicks in the <strong>3ds Max</strong> viewports and are described in <a href=\"./../Scripted-Mouse-Tools.html\">Scripted Mouse Tools</a>.</p>\n\
</li>\n\
<li><p><strong>User-interface control items</strong> are elements displayed in the rollout such as, buttons, spinners, and list boxes.</p>\n\
</li>\n\
<li><p><strong>Event handlers</strong> are special functions associated with certain user-interface control items. Event handlers specify the processing that must occur when a user interacts with a user-interface item. For example, pressing a button or adjusting a spinner. These user actions generate named events and the optional event handler you supply for that event is called when the user action occurs. For example, if you press a button named <strong>DoIt</strong>, the <strong>on DoIt pressed</strong> event handler expression is executed.</p>\n\
</li>\n\
</ul>\n\
<p>Formally, the syntax of a <code>&lt;rollout_clause&gt;</code> is defined as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;rollout_clause&gt; ::= &lt;local_variable_decl&gt; | &lt;local_function_decl&gt; | &lt;local_struct_decl&gt; | &lt;global_variable_decl&gt; | &lt;mousetool&gt; | &lt;user_interface_item&gt; | &lt;item_group&gt; | &lt;event_handler&gt;</code></pre><p><strong>Locals</strong></p>\n\
<p>A <code>&lt;local_variable_decl&gt;</code> , <code>&lt;global_variable_decl&gt;</code> , <code>&lt;local_function_decl&gt;</code> , and <code>&lt;local_struct_decl&gt;</code> are exactly the same as local and global variable, local function, and local structure definitions in MAXScript:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_variable_decl&gt; ::= local &lt;decl&gt; { , &lt;decl&gt; } \n\
&lt;decl&gt; ::= &lt;name&gt; [ = &lt;expr&gt; ] -- optional initial value</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;global_variable_decl&gt; ::= global &lt;decl&gt; { , &lt;decl&gt; } \n\
&lt;decl&gt; ::= &lt;name&gt; [ = &lt;expr&gt; ] -- optional initial value</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_function_decl&gt; ::= [ mapped ](function | fn) &lt;name&gt; { &lt;argument&gt; } = &lt;expr&gt;</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_struct_decl&gt; ::= struct &lt;name&gt; ( &lt;member&gt; { , &lt;member&gt; } )\n\
&lt;member&gt; ::= ( &lt;name&gt; [ =&lt;expr&gt; ] | &lt;local_function_decl&gt; )</code></pre><p>Examples of the previous (in order) are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local numSelected\n\
global foo\n\
local numSelected = 0\n\
fn onlyOneSelected = (selection.count == 1)\n\
struct parents (mother=&quot;&quot;, father=&quot;&quot;)\n\
</code></pre><p>When writing scripts, it is a good programming practice to explicitly declare your local and global variables. Implicit declaration is provided as a short-hand, typically used when working in the Listener interactively or developing short scripts. When developing extended scripts, explicitly declaring variables can reduce errors and improve readability of the code. It is also recommend to declare all variables as local unless you really want them to be global variables. The reasons for this are described in <a href=\"./../../../MAXScript-Language-Reference/Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a>.</p>\n\
<p><strong>User-Interface Items</strong></p>\n\
<p>A <code>&lt;user_interface_item&gt;</code> defines an individual button, check box, spinner, or other user-interface control item that will appear in the rollout. These user-interface control items are described in <a href=\"Rollout-User-Interface-Controls.html\">Rollout User-Interface Controls</a>.</p>\n\
<p>An <code>&lt;item_group&gt;</code> is used to place a sequence of user-interface items in a labeled box in the rollout, so you can organize large rollouts into meaningful groups.</p>\n\
<p><strong>Its syntax is:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">group &lt;group_label_string&gt; \n\
( \n\
{ &lt;user_interface_item&gt; } \n\
) </code></pre><p>The use of <code>group</code> is shown in the following script:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   utility Infinity &quot;Game Utilities&quot;\n\
   (\n\
   group &quot;Lighting&quot;\n\
   (\n\
   label label1d &quot;Number of Day lights:&quot; across:2 offset:[10,0]\n\
   label label2d &quot;0&quot; offset:[10,0]\n\
   label label1n &quot;Number of Night lights:&quot; across:2 offset:[13,0]\n\
   label label2n &quot;0&quot; offset:[10,0]\n\
   label label3\n\
   Radiobuttons WhichOn &quot;Active Lights:&quot; labels:#(&quot;Day&quot;,&quot;Night&quot;)\n\
   )\n\
   group &quot;Scene Data Dump&quot;\n\
   (\n\
   Button scenedump &quot;Dump Scene Data&quot;\n\
   )\n\
   group &quot;Exclusions/Inclusions&quot;\n\
   (\n\
   Button DispExcl &quot;Unhide Exclusions&amp;Inclusions&quot;\n\
   )\n\
   group &quot;Camera Mattes&quot;\n\
   (\n\
   radiobuttons CamMatte labels:#(&quot;None&quot;,&quot;1&quot;,&quot;2&quot;,&quot;3&quot;,&quot;4&quot;) columns:3\n\
   )\n\
   Button resetb &quot;Reset&quot;\n\
   )\n\
   createDialog infinity\n\
</code></pre></blockquote>\n\
<p>The Utilities panel rollout that the previous script generates looks like the following figure.</p>\n\
<p><img src=\"./images/GUID-E0CDD9A7-9855-440A-A567-A0AB1C53AEE3-low.gif\" alt></p>\n\
<p><strong>Game Utilities rollout using group user-interface item</strong></p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> A group is technically not an user-interface control. If you want to use a group that is an interface control, consider the <a href=\"Rollout-User-Interface-Controls/Rollout-User-Interface-Controls-3/GroupBox-UI-Control.html\">GroupBox</a> user-interface control.</div>\n\
<p>Internally, a group is implemented as two controls of <code>GroupStartControl</code> and <code>GroupEndControl</code> classes, enclosing the grouped user-interface controls like brackets.</p>\n\
<p>In <strong>3ds Max 8 and higher</strong>, you can access and change the <code>.caption</code> property of either controls if you access them using the <code>.controls</code> property of a rollout. This was not possible in the previous versions.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   rollout test &quot;Test&quot;\n\
   (\n\
   group &quot;Group&quot;\n\
   (\n\
   button pressme &quot;Press Me&quot;\n\
   )\n\
   )\n\
   createDialog test\n\
   test.controls\n\
   test.controls[1].caption = &quot;XXX&quot; --this is the GroupStartControl\n\
   test.controls[3].caption = &quot;Test me&quot; --this is the GroupEndControl\n\
   --Both change the group&apos;s caption!\n\
</code></pre></blockquote>\n\
<p>In <strong>3ds Max 2010 and higher</strong>, user Interface controls&apos; event handlers can be included inside a Group&apos;s body. In previous versions of <strong>3ds Max</strong>, including an even handler inside the group caused a Syntax Error.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   rollout test &quot;Test&quot;\n\
   (\n\
   group &quot;Group&quot;\n\
   (\n\
   button pressme &quot;Press Me&quot;\n\
   --In versions prior to 3ds Max 2010,\n\
   --this would have caused a Syntax Error:\n\
   on pressme pressed do print &quot;Pressed!&quot;\n\
   ) --because the handler is inside the group&apos;s body.Now allowed.\n\
   )\n\
   createDialog test\n\
</code></pre></blockquote>\n\
<p><strong>Event Handlers</strong></p>\n\
<p>An <code>&lt;event_handler&gt;</code> is a special function definition local to a utility or rollout that you provide to handle the processing that must occur when a user performs an action on a user-interface item. For example, event handlers are called when the user presses a button or adjusts a spinner, opens or closes the utility or rollout, or resizes or moves a rollout floater window. These user actions generate named events and any event handler you supply for that event is called when the action occurs. The syntax for defining an event handler is as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on &lt;item_name&gt; &lt;event_name&gt; [ &lt;argument&gt; ] do &lt;expr&gt;</code></pre><p>The <code>&lt;item_name&gt;</code> specifies the name of the item to which this handler is connected. The <code>&lt;event_name&gt;</code> specifies the type of event to be handled and the optional <code>&lt;argument&gt;</code> is used to pass various values to the handler. The possible events you can specify depend on the item type. These events include:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">pressed changed picked entered selected resized moved open close</code></pre><p>The available events and passed arguments are defined in the description for each user-interface item type, as listed in <a href=\"Rollout-User-Interface-Controls.html\">Rollout User-Interface Controls</a>. The available events and passed arguments for utilities and rollouts are described in <a href=\"Utility-and-Rollout-Properties.html\">Utility and Rollout Properties, Methods, and Event Handlers</a>.</p>\n\
<p>You can access and invoke the event handler functions in a scripted rollout by referring to the handlers as properties on the user-interface items. The event handler functions are accessed as sub-properties of the item using the event name as the sub-property name. For example, if a scripted rollout has a check box item named <code>foo</code> , and an <code>on foo changed</code> event handler is defined, you can invoke the handler as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">foo.changed true -- call foo&apos;s &apos;changed&apos; handler function, passing argument of true</code></pre><p>Or, if a scripted rollout has a button item named <code>apply</code> , and an <code>on apply pressed</code> event handler is defined, you can invoke the handler as follows:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   apply.pressed() -- call apply&#x2019;s &apos;pressed&apos; handler function, no argument.\n\
</code></pre></blockquote>\n\
<p>You can access an event handler function as a sub-property of the item.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   ApplyPressedEH = apply.pressed\n\
</code></pre><p>   stores a copy of the <code>on apply pressed</code> event handler function value to variable <code>ApplyPressedEH</code> . The event handler functions can only be <strong>read</strong>. You <strong>cannot set</strong> the handler function to another user-defined function.</p>\n\
</blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Utilities-and-Rollouts.html\">Scripted Utilities and Rollouts</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";