var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-1B088FF0-6A36-420E-9F37-F0DBE9FB2676\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Viewport Drawing Methods</h1></div><p>The methods documented in this topic provide low-level access to <strong>3ds Max</strong>&apos;s graphics system. These methods are available for scripts to do any graphics work not possible using the standard high-level graphics methods.</p>\n\
<p>These methods are for use in the existing <strong>3ds Max</strong> viewports, and only work on the active viewport. Note that these methods typically are not for casual use, as they are not intended to be a high level graphics library. For example, many steps are required to display a single lit polygon. These methods are optimized for speed, and not at all for script programmer ease of use.</p>\n\
<p>The methods described in this topic actually operate on graphic windows. While graphic windows are not the same as viewports, they are related to one another. Each viewport has its own graphics window, and the contents of the viewport display can be thought of as a snapshot of the graphics window contents. Since writing to display memory a relatively slow operation, <strong>3ds Max</strong> writes instead to the graphics window, and then when all the writes have been finished, it redraws the viewports with the graphics window contents.</p>\n\
<p>Many of these methods take an optional <code>applyUIScaling</code> argument, which always defaults to true. This argument specifies whether to apply any scaling applied by the system on high-DPI displays.</p>\n\
<h2 id=\"viewport-drawing-and-the-nitrous-graphics-manager\">Viewport Drawing and the Nitrous Graphics Manager</h2><p>The Nitrous Graphics Manager introduced in <strong>3ds Max 2012</strong> performs continuous updates to refine the final image. This requires special handling of all Graphics Methods described further in this topic. These Methods will only be executed if they are wrapped in a Scene Redraw Callback function.</p>\n\
<p>In other words, existing scripts will have to be modified to check for the Nitrous Graphics Manager if running <strong>3ds Max 2012 or higher</strong> and call all viewport drawing operations from a Scene Redraw Callback function, and newly developed scripts must take this into account and implement the same approach.</p>\n\
<p>The MacroScript <em>Macro_GrabViewport.mcr</em> shipping with <strong>3ds Max</strong> has been updated according to these rules and can be used as an example.</p>\n\
<p>Here is another example:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<p>   The following script will display the class, name and position of every object in the scene:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   unregisterRedrawViewsCallback GW_displayObjectNames\n\
   fn GW_displayObjectNames =\n\
   (\n\
     gw.setTransform (matrix3 1)\n\
     for o in objects where not o.isHiddenInVpt do\n\
       gw.text o.pos (o as string) color:yellow\n\
     gw.enlargeUpdateRect #whole  \n\
   )\n\
   registerRedrawViewsCallback GW_displayObjectNames\n\
</code></pre></blockquote>\n\
<p>While the above example would also work in older versions of <strong>3ds Max</strong> and using the Direct3D driver, this is the ONLY way to implement gw Viewport Drawing methods when using the Nitrous Graphics Manager in <strong>3ds Max 2012</strong>.</p>\n\
<h2 id=\"graphics-driver-configuration-and-support-methods\">Graphics Driver Configuration and Support Methods</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getDriverString()         </code></pre><p>This method returns a string identifying the graphics driver (and includes manufacturer info if available)</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<p>   on an nVidia card using MAXtreme drivers, the result would look like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.getDriverString()\n\
   --&gt;&quot;NVIDIA MAXtreme v. 6.00.07&quot;\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.querySupport &lt;feature_name&gt;      </code></pre><p>Determines whether the driver supports the specified feature. The valid <code>&lt;feature_name&gt;</code> values are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#txtCorrect </code></pre><p>Determines whether the driver supports texture correction.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#geomAccel </code></pre><p>This is used to indicate to <strong>3ds Max</strong> (and the mesh class in particular) that the driver wants to handle all of the 3D data natively. In this case, meshes are rendered by passing 3D world space data and letting the driver transform, clip, and light the vertices. If this returns <code>false</code> , then the mesh class handles all transforms, clipping and lighting calculations and then calls the hPolygon or hPolyline 2 1/2D calls for the driver to rasterize. (Primitives that are actually clipped are still sent to the polygon/polyline methods.)</p>\n\
<p>Currently, only the OpenGL driver returns <code>true</code> to this query, but other drivers have been developed that return <code>true</code> , and the HEIDI and D3D drivers may change in the future.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#triStrips </code></pre><p>If this returns <code>true</code> , then <strong>3ds Max</strong> will try to stripify meshes before calling the rendering methods. Currently, the drivers just return the user preference that is set in the driver configuration dialog. This preference defaults to <code>true</code> .</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#dualPlanes </code></pre><p>If a driver has dual-planes support it returns <code>true</code>. The standard <strong>3ds Max</strong> OpenGL display driver only returns <code>true</code> for this if the underlying display driver has implemented a custom OpenGL extension that allows <strong>3ds Max</strong> to handle this efficiently.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#swapModel </code></pre><p>This returns <code>true</code> if <strong>3ds Max</strong> has to redraw the whole scene any time the viewports are exposed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#incrUpdate </code></pre><p>This returns <code>true</code> if the driver can update a rectangular subset of the viewport without trashing the image outside that rectangle. This is <code>true</code> for most drivers that blit the viewport region and <code>false</code> for those that do page-flipping in the hardware. For OpenGL, this is <code>true</code> if the display driver implements the Microsoft <code>glSwapRectHintWIN</code> extension.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#passDecal </code></pre><p>This is <code>true</code> if the driver can handle de-calling with only one pass. Currently, this is <code>true</code> for OpenGL, but <code>false</code> for HEIDI and D3D.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#driverConfig </code></pre><p>This is <code>true</code> if the driver has a configuration dialog. This is <code>true</code> for all three of <strong>3ds Max</strong>&#x2019;s standard drivers.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#texturedBkg </code></pre><p>This is <code>true</code> if the viewport background is implemented as a textured rectangle, and <code>false</code> if it is a blitted bitmap.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#virtualVpts </code></pre><p>This is <code>true</code> if the driver allows viewports to be made larger than the physical window they are attached to. Currently, this is only <code>true</code> for OpenGL.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#paintDoesBlit </code></pre><p>This is <code>true</code> if WM_PAINT messages result in a blit of the backbuffer (as opposed to a page-flipping swap). This allows <strong>3ds Max</strong> to do quick damage region repair, and works together with the <code>#swapModel</code> flag.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#wireframeStrips </code></pre><p>This is <code>true</code> if the driver wants <strong>3ds Max</strong> to send down wireframe models using triangle strips instead of a bundle of 2-point segments. This is only used by the OpenGL driver, and it is there as a user-choosable performance-accuracy tradeoff (since the strips are faster and are back-culled, but they display hidden edges as though they are visible).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.dualPlane </code></pre><p>Gets/Sets the status of the Viewport Parameters/Use Dual Planes checkbox in the Preference Settings dialog, Viewports tab.</p>\n\
<h2 id=\"window-and-viewport-transformation-methods\">Window and Viewport Transformation Methods</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.isPerspectiveView() </code></pre><p>Returns <code>true</code> if the view is in perspective projection; otherwise <code>false</code> (orthographic projection).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setTransform &lt;matrix3&gt; </code></pre><p>Sets the active viewport&#x2019;s graphics window transformation matrix to the specified matrix3 value, and updates the modeling coordinates to normalized projection coordinates matrix. This routine also back-transforms each light and the eye point so that lighting can be done in modeling coordinates.</p>\n\
<p>This method may be used to set a matrix that transforms the point passed to the drawing methods (like <code>gw.text()</code> , <code>gw.marker()</code> , <code>gw.polyline()</code> or <code>gw.polygon()</code> ). Normally these methods expect world coordinates. However if this matrix is set to an object&#x2019;s transformation matrix you can pass coordinates in the object&#x2019;s space coordinates and they will be transformed into world space (and then put into screen space when they are drawn). If however this is set to the identity matrix, you would pass world space coordinates. You can set this matrix to the object&#x2019;s transform matrix using the following</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.setTransform node.transform\n\
</code></pre></blockquote>\n\
<p>For world-to-screen space conversions by the methods <code>gw.text()</code> , <code>gw.marker()</code> , <code>gw.polyline()</code> or <code>gw.polygon()</code> , etc, a developer must explicitly set this matrix to the identity matrix. This is because the graphics window may have a non-identity transform matrix already in place from a previous operation.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.setTransform (matrix3 1)\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getFlipped() </code></pre><p>Returns <code>true</code> if the determinant of the current transform is positive, <code>false</code> if negative.</p>\n\
<h2 id=\"position-size-and-depth-clipping-methods\">Position, Size, and Depth Clipping Methods</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setPos &lt;x_integer&gt; &lt;y_integer&gt; &lt;w_integer&gt; &lt;h_integer&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Sets the size and position of the graphics window. The coordinates are all Windows coordinates in the space of the graphics windows&apos; parent window. All coordinates are in Windows format, with the origin in the upper left.</p>\n\
<p>x - Specifies the left graphics window origin; y - Specifies the top graphics window origin; w - Specifies the graphics window width; h - Specifies the graphics window height.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getWinSizeX() [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method gets the current window size in X.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getWinSizeY() [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method gets the current window size in Y.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getWinDepth() </code></pre><p>This method returns the z-buffer depth (in bits).</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This method does not return the proper Z depth value in <strong>3ds Max</strong>.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getHitherCoord() </code></pre><p>This method returns the largest device Z value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getYonCoord() </code></pre><p>This method returns the smallest device Z value.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> This method does not return the proper Z depth value in <strong>3ds Max</strong>.</div>\n\
<p><a name=\"dib--device-independent-bitmap--methods\"></a></p>\n\
<h2 id=\"dib-device-independent-bitmap-methods\">DIB (Device-Independent Bitmap) Methods</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getViewportDib captureAlpha:&lt;boolean&gt; gammaCorrect:&lt;boolean&gt; </code></pre><p>This method returns the active viewport&#x2019;s graphics window image as a Bitmap value.</p>\n\
<p>The size of the bitmap is the same size as the viewport.</p>\n\
<p>Since <strong>3ds Max 2015</strong>, if the <code>captureAlpha:</code> optional keyword argument is specified and set to True, the viewport Alpha will also be captured. If set to False or not specified, the returned image will contain no Alpha and only the RGB channels.</p>\n\
<p>Moreover, since <strong>3ds Max 2015</strong> if the <code>gammaCorrect:</code> optional keyword argument is specified and set to False, no Gamma correction will be performed on the bitmap value. If set to True or not specified, the returned image will be Gamma corrected.</p>\n\
<p>See also <a href=\"Accessing-Active-Viewport-Info.html\"\">Viewport.GetViewportDib()</a> and <a href=\"./../../../Interaction-With-The-Operating/Windows-Structure.html\"\">windows.snapshot()</a>.</p>\n\
<h2 id=\"drawing-setup\">Drawing Setup</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.resetUpdateRect() </code></pre><p>This method resets the update rectangle. The update rectangle is the region of the screen that needs to be updated to reflect items that have changed. When the system is done rendering items, the goal is to only update the region of the viewport that has actually been altered. This method sets the update rectangle (the region that will be blitted to the display) to a special &quot;empty&quot; value. This way when <code>gw.enlargeUpdateRect()</code> is later called, the Box2 region passed will be used as the region.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.enlargeUpdateRect ( &lt;Box2&gt; | #whole ) [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method enlarges the update rectangle to include the Box2 value passed. If <code>#whole</code> is specified, the whole viewport will later be updated</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getUpdateRect() [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method retrieves the current update rectangle as a Box2 value. Returns a special an Box2 &quot;empty&quot; value if no region of the viewport needs to be updated.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setRndLimits &lt;render_limits_name_array&gt; </code></pre><p>Sets the rendering limits used by primitive calls. Setting the rendering limits is used in communication between the various parts of <strong>3ds Max</strong> that handle the display of objects. For example, setting this limit to #polyEdges and then drawing a polygon won&apos;t result in a polygon drawn with edges. It only sets a flag that indicates the edge should be drawn.</p>\n\
<p>What happens is as follows. Inside the upper level <strong>3ds Max</strong>, part of the code knows that polygon edges have been turned on. However this is not related through the object oriented architecture to the part of <strong>3ds Max</strong> that does the actual drawing. When <strong>3ds Max</strong> goes to draw objects it will see that the polygon edge flag is on. This tells it to do two drawing passes -- one to draw the polygon, then it calls <code>outlinePass()</code> call with <code>true</code> , draws a bunch of edges, then calls <code>outlinePass()</code> with <code>false</code> . Thus, the drawing routine is responsible for looking at the flags and drawing appropriately. This method is only responsible setting the limit which can later be checked.</p>\n\
<p><code>&lt;render_limits_name_array&gt;</code> specifies the rendering limits used by the viewport as an array of <code>&lt;render_limits_name&gt;</code> values. The valid <code>&lt;render_limits_name&gt;</code> values are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#allEdges </code></pre><p>All edges of the item are shown (including hidden ones).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#boxMode </code></pre><p>Objects are shown using their bounding box.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#backcull </code></pre><p>Backface culling is used. Entities whose surface normal face away from the view direction are not drawn.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#colorVerts </code></pre><p>This turns on color-per-vertex display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#flat </code></pre><p>Flat (facet) shading mode.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#illum </code></pre><p>This indicates that you have colors per vertex in your polygons and that they should be used. If you had colors per vertex but this flag was not set, the colors would be ignored.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#lighting </code></pre><p>This is the same as setting <code>#illum</code> and <code>#specular</code> .</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#noAtts </code></pre><p>No attributes are specified.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#perspCorrect </code></pre><p>In this mode textures are corrected for perspective display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#pick </code></pre><p>This indicates hit testing will be performed (not rendering).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#polyEdges </code></pre><p>This mode causes polygon edges (Edged Faces) to be on.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#shadeCverts </code></pre><p>This modifies <code>#colorVerts</code> . If set, lighting is enabled and the vertex colors are used to modulate the colors that result from lighting. If off, the colors on each vertex are used directly to shade the triangle. When <strong>3ds Max</strong> uses <code>#shadeCverts</code> mode, it puts a white diffuse-only material on the object so that it appears that the colors are shaded without distortion.</p>\n\
<p>Described further, when <code>#shadeCverts</code> is OFF, then the vertex colors are used directly. This is equivalent to saying that they are modulated by a pure white self-illuminated material.</p>\n\
<p>When <code>#shadeCverts</code> is ON, the diffuse white material is illuminated by the scene lighting, resulting in shades ranging from black to white, with most vertices being some shade of pure gray. When the vertex colors are modulated by the material color, they get multiplied (in general) by a number less than 1, which makes them appear darker.</p>\n\
<p>The RGB components of the colors are modulated uniformly, so that there is no shift from, say, red to green. That would happen if the underlying material was not evenly weighted (that is, a pure gray lying between black and white). Said another way, only the intensity of the vertex colors is changed when shading is on, not luminance, chrominance, etc.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#specular </code></pre><p>This enables specular highlight display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#texture </code></pre><p>This enables texture display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#twoSided </code></pre><p>Faces are displayed regardless of their surface normal orientation.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#vertTicks </code></pre><p>This mode is really a pseudo-mode, in that it doesn&apos;t actually cause the graphics drivers to do anything differently, but rather is tested by the Mesh class, which sends down vertex markers (+) if the mode is on.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#wireframe </code></pre><p>Wireframe rendering mode.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#zBuffer </code></pre><p>When coordinates are specified for drawing primitives they have x, y, and z values. Sometimes when drawing entities in the viewports it is desirable to ignore the z values. For example in the <strong>3ds Max</strong> viewports the text that display the type of viewport (Front, Left, and so on) are drawn without z values. So are the arc-rotate circle control and the axis tripods. These items are drawn without this flag being set so they always show up in front.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getRndLimits() </code></pre><p>Retrieves the rendering limits used by primitive calls as an array of names. See <code>gw.setRndLimits()</code> for a list of the returned name values.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getRndMode() </code></pre><p>Returns the current rendering mode used by the viewport as an array of names. This is a subset of the rendering limit, in that any limits imposed by the rendering limit are forced onto the current mode. See <code>gw.setRndLimits()</code> for a list of the returned name values.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setSkipCount &lt;skip_count_integer&gt; </code></pre><p>Sets the number of triangles skipped when the viewport is set as a &apos;Fast View Display&apos; viewport. To disable fastview, specify 1. Since triangles are handed down to graphics driver one at a time, it is up to the code that feeds triangles to the graphics driver to skip the specified number of triangles. The mesh rendering in <strong>3ds Max</strong> uses the skip count in this way.</p>\n\
<p><code>&lt;skip_count_integer&gt;</code> specifies that every &apos;n-th&apos; triangle should be drawn. If set to 2, every other triangle should be drawn.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getSkipCount()</code></pre><p>Returns the current skip count setting.</p>\n\
<blockquote>\n\
<p>   <strong>The following is an example of using the above functions</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   -- Get the current rendering limits\n\
   lim = gw.getRndLimits()\n\
   -- Add another limit\n\
   append lim #polyEdges\n\
   -- Set the new rendering limits\n\
   gw.setRndLimits lim\n\
   -- Get back the rendering limits to check\n\
   gw.getRndLimits()\n\
   -- Get back the rendering mode to check\n\
   gw.getRndMode()\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setDirectXDisplayAllTriangle &lt;boolean&gt; </code></pre><p>Exposes the <strong>Display All Triangle Edges</strong> option available in the UI as a checkbox in the <strong>Customize&gt;Preferences&gt;Viewports&gt;Configure Driver</strong> dialog of the Direct3D driver. When set to true, all edges will be drawn. When set to false, only visible edges will be drawn.</p>\n\
<p>Available in <strong>3ds Max 9</strong> and higher.</p>\n\
<h2 id=\"light-and-camera-methods\">Light and Camera Methods</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getMaxLights() </code></pre><p>Returns the maximum number of lights that may be used by the interactive renderer.</p>\n\
<h2 id=\"coordinate-transformation-methods\">Coordinate Transformation Methods</h2><p>The following methods map points from the graphic window&apos;s current transform to device space. If the graphic window&apos;s transform is set to the identity matrix then the mapping is done from points specified in world space. Otherwise the points given are transformed by the graphic window&#x2019;s transform, and are then considered to be in world space. Thus, o get a world-space to screen-space conversion, you need to set the graphic window&#x2019;s transform to the identity with</p>\n\
<blockquote>\n\
<p>   <strong>CODE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.setTransform(Matrix3 1)\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.hTransPoint &lt;point3&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method converts the point3 coordinate to a &quot;h&quot; format device coordinate. Each component of the return value is in integer format in the native device coordinates for the graphics driver. For HEIDI and OpenGL, the origin is at the lower left. For Direct3D the origin is at the upper left.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.wTransPoint &lt;point3&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method converts the point3 coordinate to a &quot;w&quot; format device coordinate. Each component of the return value is in integer format with the origin at the upper left.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.transPoint &lt;point3&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method converts the point3 coordinate to a &quot;h&quot; floating point coordinate. Each component of the return value is in float format with the origin at the upper left. This is just a helper routine to avoid building up round-off error. <strong>3ds Max</strong> uses it just for IK.</p>\n\
<h2 id=\"drawing-methods\">Drawing Methods</h2><p>Methods that start with &quot;h&quot; take integer device coordinates with the origin at the lower-left. Methods that start with &quot;w&quot; in front take Windows device coordinates with the origin at the upper left. These &quot;h&quot; and &quot;w&quot; routines perform NO clipping unless otherwise noted. Drawing outside the allowable region is likely to cause <strong>3ds Max</strong> to crash. These coordinate systems are left-handed.</p>\n\
<p>Methods that don&#x2019;t start with &quot;h&quot; or &quot;w&quot; map points from the graphic window&apos;s current transform to device space. This coordinate system is right-handed. If the graphic window&apos;s transform is set to the identity matrix then the mapping is done from points specified in world space. Otherwise the points given are transformed by the graphic window&#x2019;s transform, and are then considered to be in world space. Thus, to get a world-space to screen-space conversion, you need to set the graphic window&#x2019;s transform to the identity with:</p>\n\
<blockquote>\n\
<p>   <strong>CODE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.setTransform(Matrix3 1)\n\
</code></pre></blockquote>\n\
<p>After completing any drawing to the graphics window with the methods described in this section, you need to call <code>gw.updateScreen()</code> to update the viewport display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.updateScreen() </code></pre><p>Updates the viewport display to display any text, markers, polylines, polygons, or tristrips written to the graphics window via the methods described below.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.text &lt;point3&gt; &lt;string&gt; [ color:&lt;color&gt; ]\n\
gw.hText &lt;point3&gt; &lt;string&gt; [ color:&lt;color&gt; ] [ applyUIScaling:&lt;true&gt; ]\n\
gw.wText &lt;point3&gt; &lt;string&gt; [ color:&lt;color&gt; ] [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Draws 2D fixed font annotation string text to the specified location using the specified (optional) color. If the color is not specified, a default color of red is used.</p>\n\
<p>Multi-line text is supported since <strong>3ds Max 2015</strong> - use the \\n escape sequence to introduce New Lines into the string. In previous versions of <strong>3ds Max</strong>, only single-line text was supported.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>This routine DOES perform clipping of the text if it is off the screen.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.getTextExtent &lt; string &gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Returns the text extents of the string as it would be displayed in the viewports as a point2 value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.Marker &lt;point3&gt; &lt;marker_name&gt; [ color:&lt;color&gt; ] \n\
gw.hMarker &lt;point3&gt; &lt;marker_name&gt; [ color:&lt;color&gt; ] [ applyUIScaling:&lt;true&gt; ]\n\
gw.wMarker &lt;point3&gt; &lt;marker_name&gt; [ color:&lt;color&gt; ] [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Draws a marker at the specified location. This is can be paired with <code>pickpoint()</code> to quickly show where the user has clicked. These markers are temporary and will be erased whenever the viewports are updated. If the color is not specified, a default color of red is used. The valid <code>&lt;marker_name&gt;</code> types are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#point #hollowBox #plusSign #asterisk #xMarker #bigBox #circle #triangle #diamond #smallHollowBox #smallCircle #smallTriangle #smallDiamond </code></pre><p>The following is an example that creates an instance of all types of markers, spaced equally:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   arr = #(&quot;point&quot;,&quot;hollowBox&quot;,&quot;plusSign&quot;,&quot;asterisk&quot;,&quot;xMarker&quot;,\n\
   &quot;bigBox&quot;,&quot;circle&quot;,&quot;triangle&quot;,&quot;diamond&quot;,&quot;smallHollowBox&quot;,\n\
   &quot;smallCircle&quot;,&quot;smallTriangle&quot;,&quot;smallDiamond&quot;)\n\
   for i=1 to arr.count do (\n\
   gw.hMarker [100, (50 + i*10), 50](arr[i] as name) )\n\
   gw.enlargeUpdateRect #whole\n\
   gw.updateScreen()\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.Polyline &lt;vertex_point3_array&gt; &lt;isClosed_boolean&gt; [rgb:&lt;color_array&gt;] \n\
gw.hPolyline &lt;vertex_point3_array&gt; &lt;isClosed_boolean&gt; [rgb:&lt;color_array&gt;] [ applyUIScaling:&lt;true&gt; ] \n\
gw.wPolyline &lt;vertex_point3_array&gt; &lt;isClosed_boolean&gt; [rgb:&lt;color_array&gt;] [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method draws a multi-segment polyline. Each value in <code>&lt;vertex_point3_array&gt;</code> is a vertex on the polyline. If <code>&lt;isClosed_boolean&gt;</code> is <code>true</code> , the first point is connected to the last point, that is, the polyline is closed. If <code>false</code> , the polyline is left open. If the optional rgb color array is specified, and shade mode is set to smooth, the polyline will be drawn Gourand shaded. This is how <strong>3ds Max</strong> draws lit wireframes for instance. If the optional rgb color array is not specified, the line is drawn with the line color specified via <code>gw.setColor()</code> . The number of elements in <code>&lt;color_array&gt;</code> must be the same as in <code>&lt;vertex_point3_array&gt;</code>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.Polygon &lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; \n\
gw.hPolygon &lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; [ applyUIScaling:&lt;true&gt; ]\n\
gw.wPolygon &lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>This method draws a multi-point polygon. Each value in <code>&lt;vertex_point3_array&gt;</code> is a vertex on the polygon. <code>&lt;color_array&gt;</code> specifies the color at each vertex. The rendering mode (set via <code>gw.setRndLimits()</code> ) must include <code>#colorVerts</code> for the color values to be used. <code>&lt;uvw_point3_array&gt;</code> specifies the UVW coordinates at each. The rendering mode must include <code>#texture</code> for the UVW coordinates to be used. The number of elements in each array must be identical.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.hRect &lt;box2&gt; &lt;color&gt; [ applyUIScaling:&lt;true&gt; ]\n\
gw.wRect &lt;box2&gt; &lt;color&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Draws a rectangle in the given color using the coordinates specified by box2.</p>\n\
<p>Available in <strong>3ds Max 2008</strong> and higher. reviously available in <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   rect = (box2 13 47 96 97)\n\
   gw.wrect rect red\n\
   gw.wmarker [rect.left,rect.top,0] #point color:green\n\
   gw.wmarker [rect.left,rect.bottom,0] #point color:green\n\
   gw.wmarker [rect.right,rect.top,0] #point color:green\n\
   gw.wmarker [rect.right,rect.bottom,0] #point color:green\n\
   eRect = rect -- enlargeUpdateRect 1 pixel too small in either direction?\n\
   eRect.right += 1\n\
   eRect.bottom += 1\n\
   gw.enlargeUpdateRect eRect\n\
   gw.updateScreen()\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.startTriangles() </code></pre><p>This method is called to begin sending a series of triangles to render. Call this method, then <code>gw.triangle()</code> one or more times, then <code>gw.endTriangles()</code> to finish.</p>\n\
<p>Available in <strong>3ds Max 2008</strong> and higher. Previously available in <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.endTriangles() </code></pre><p>Call this method to finish rendering triangles.</p>\n\
<p>Available in <strong>3ds Max 2008</strong> and higher. reviously available in <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.triangle &lt;vertex_point3_array&gt; &lt;color_array&gt; </code></pre><p>This method draws a triangle. Each value in <code>&lt;vertex_point3_array&gt;</code> is a vertex on the triangle.</p>\n\
<p><code>&lt;color_array&gt;</code> specifies the color at each vertex. The rendering mode set via <code>gw.setRndLimits()</code> must include <code>#illum</code> for the color values to be used.</p>\n\
<p>There must be 3 elements in each array.</p>\n\
<p>Call <code>gw.startTriangles()</code> first, call <code>gw.triangle()</code> one or more times, then <code>gw.endTriangles()</code> when done drawing triangles.</p>\n\
<p>Available in <strong>3ds Max 2008</strong> and higher. reviously available in <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   gw.setTransform(Matrix3 1)\n\
   gw.startTriangles()\n\
   gw.triangle #([0,0,0], [50,0,0], [50,50,0]) #(red,red,red)\n\
   gw.triangle #([0,0,0], [50,50,0], [0,50,0]) #(green,green,green)\n\
   gw.endTriangles()\n\
   gw.enlargeUpdateRect #whole\n\
   gw.updateScreen()\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.triStrip&lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; </code></pre><p>This method is used for drawing a series of triangles specified as &apos;strips&apos;.</p>\n\
<p>The method takes 3 or more vertices and builds triangles in a strip. This sends a lot less data and the underlying graphics library has to set up a lot less data since it can use the previous information to start the rasterization, resulting in a significant speed increase. This routine does no clipping so all the vertices passed must be within view.</p>\n\
<p>The parameters are the same as for the <code>gw.Polygon()</code> . However, the <code>&lt;vertex_point3_array&gt;</code> is handled differently. After the first two vertices, each new vertex is used to create a new triangle. For instance, to draw a quad, the first three vertices specify the first triangle and the next one is combined with the previous two to complete the square.</p>\n\
<p>Available in <strong>3ds Max 2008</strong> and higher. Previously available in <a href=\"./../../../MAXScript-Introduction/MAXScript-for-New-and-Casual/Avguard-DLX-Extension.html\">Avguard Extensions</a>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.hTriStrip &lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; [ applyUIScaling:&lt;true&gt; ]\n\
gw.wTriStrip &lt;vertex_point3_array&gt; &lt;color_array&gt; &lt;uvw_point3_array&gt; [ applyUIScaling:&lt;true&gt; ]</code></pre><div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>These two methods are currently broken. Please use the gw.triStrip and Polygon methods above.</p>\n\
<p>The following is an example of using the above functions:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   -- Draw some primitives\n\
   gw.hPolyline #([300,50,16], [300,200,8], [450,250,4]) true\n\
   --\n\
   gw.hPolygon #([200,100,16], [280,100,8], [250,200,4]) \\\n\
   #(red, blue, green) \\\n\
   #([1.0,.5,0], [0.5,0.5,0], [0,0,0.5])\n\
   -- Update the viewports\n\
   gw.enlargeUpdateRect #whole\n\
   gw.updateScreen()\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.setColor &lt;type_name&gt; &lt;color_value&gt; </code></pre><p>Sets the RGB color used for the specified drawing type. The valid <code>&lt;type_name&gt;</code> values are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">#line -- line drawing color \n\
#fill -- polygon fill color \n\
#text -- text drawing color \n\
#clear -- The color that the viewport is cleared to when you call gw.clearScreen() </code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">gw.clearScreen &lt;Box2&gt; [ useBkg:&lt;boolean&gt; ] [ applyUIScaling:&lt;true&gt; ]</code></pre><p>Clears the specified rectangular region of the screen. If the optional <code>useBkg</code> parameter is set to <code>false</code> , the region is set to the &quot;clear&quot; color (see <code>gw.setColor()</code> above). If <code>true</code> , the background should be used to fill the cleared area. The default <code>useBkg</code> value is <code>false</code> .</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Viewports.html\">Viewports</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";